from .base import *

class commands(object):
    """
    cytoscape commands as shown in CyREST's swagger documentation for 'Commands'.

    :param url: an url of the type 'http://' + host + ':' + str(port) + '/' + version + '/'.
    """

    def __init__(self, url):
        self.__url = url + 'commands'

    def list_namespaces(self, verbose=False):
        """
        List all available command namespaces.
        """
        response=api(url=self.__url,method="HTML", verbose=verbose)
        return response

    def namespace(self, namespace, verbose=False):
        """
        List all available commands in a namespace.

        :param namespace: a namespace as listed in rest.cyclient.commands.list().
        :param verbose: print more
        """
        response=api(url=self.__url+"/"+namespace,method="HTML",verbose=verbose)
        return response

    def command(self, namespace, command, verbose=False):
        """
        List the arguments of a command.

        :param namespace: a namespace as listed in rest.cyclient.commands.list().
        :param command: a commands as listed in rest.cyclient.commands.namespace(<namespace>).
        :param verbose: print more
        """
        response=api(url=self.__url+"/"+namespace+"/"+command,method="HTML",verbose=verbose)
        return response

    


# -*- coding: utf-8 -*-
import json
import requests

from . import BASE_URL, HEADERS

BASE_URL_NETWORK = BASE_URL + 'networks'


class BaseView(object):
    """
    Base view class to directly access node/edge view properties.
    """

    def __init__(self, network_view=None, obj_id=None, obj_type=None):
        if network_view is None:
            raise ValueError('Network view is required.')

        if obj_type is None:
            raise ValueError('Object type is required.')

        if obj_id is None:
            raise ValueError('Object SUID is required.')
        else:
            model_id = network_view.get_model_id()
            view_id = network_view.get_id()
            self.url = BASE_URL_NETWORK + '/' + str(model_id) + \
                       '/views/' + str(view_id) + \
                       '/' + obj_type + '/' + str(obj_id)
            self.__id = obj_id

    def get_id(self):
        """Get SUID of the object

        Note that this is node/edge SUID, NOT node view/edge view SUID.

        :return: SUID as integer
        """
        return self.__id

    def set_value(self, visual_property, value):
        """Set a single Visual Property Value

        :param visual_property: Visual Property ID
        :param value: New value for the VP
        :return: None
        """
        if visual_property is None or value is None:
            raise ValueError('Both VP and value are required.')

        new_value = [
            {
                'visualProperty': visual_property,
                "value": value
            }
        ]
        requests.put(self.url, data=json.dumps(new_value), headers=HEADERS)

    def set_values(self, values):
        """
        Set multiple Visual properties at once.

        :param values:
        :return:
        """
        if values is None:
            raise ValueError('Values are required.')

        new_values = []
        for vp in values.keys():
            new_val = {
                'visualProperty': vp,
                'value': values[vp]
            }
            new_values.append(new_val)

        requests.put(self.url, data=json.dumps(new_values), headers=HEADERS)

    def get_value(self, visual_property):
        """Get a value for the Visual Property

        :param visual_property:
        :return:
        """
        res = requests.get(self.url + '/' + visual_property)
        return res.json()['value']

    def get_values(self):
        """Get all visual property values for the object

        :return: dictionary of values (VP ID - value)
        """

        results = requests.get(self.url).json()
        values = {}
        for entry in results:
            values[entry['visualProperty']] = entry['value']
        return values



from .base import *

class idmapper(object):
    """
    cytoscape network interface as shown in CyREST's swagger documentation for 'idmapper'.

    :param url: an url of the type 'http://' + host + ':' + str(port) + '/' + version + '/'.
    """

    def __init__(self, url):
        self.__url = url + 'commands/idmapper'

    def map_column(self, only_use_one=None, source_column=None, species=None, target_selection= None, verbose=False):
        """
        Uses the BridgeDB service to look up analogous identifiers from a wide
        selection of other databases

        :param only_use_one (string, optional): When multiple identifiers can be
            mapped from a single term, this forces a singular result
        :param source_column (string): Specifies the column nmae where the
            source identifiers are located = ['']
        :param source_selection (string): Specifies the database describing
            the existing identifiers = ['']
        :param species (string, optional): The combined common or latin name of
            the species to which the identifiers apply = ['Human (Homo sapiens)',
            'Mouse (Mus musculus)', 'Rat (Rattus norvegicus)', 'Frog (Xenopus tropicalis)',
            'Zebra fish (Danio rerio)', 'Fruit fly (Drosophila melanogaster)',
            'Mosquito (Anopheles gambiae)', 'Arabidopsis thaliana (Arabidopsis thaliana)',
            'Yeast (Saccharomyces cerevisiae)', 'E. coli (Escherichia coli)',
            'Tuberculosis (Mycobacterium tuberculosis)', 'Worm (Caenorhabditis elegans)']
        :param target_selection (string): Specifies the database identifiers to be looked up = ['']
        :param verbose: print more

        :returns: eg. { "new column": "SGD " }
        """
        PARAMS=set_param(["only_use_one","source_column","species","target_selection"],[only_use_one,source_column,species,target_selection])
        response=api(url=self.__url+"/map column", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response




from .base import *

class cybrowser(object):
    """
    cytoscape cybrowser interface as shown in CyREST's swagger documentation for 'cybrowser'.

    :param url: an url of the type 'http://' + host + ':' + str(port) + '/' + version + '/'.
    """

    def __init__(self, url):
        self.__url = url + 'commands/cybrowser'

    def dialog(self=None, wid=None, text=None, title=None, url=None, debug=False, verbose=False):
        """
        Launch and HTML browser in a separate window.

        :param wid: Window ID
        :param text: HTML text
        :param title: Window Title
        :param url: URL
        :param debug: Show debug tools. boolean
        :param verbose: print more
        """

        PARAMS=set_param(["id","text","title","url","debug"],[wid,text,title,url,debug])
        response=api(url=self.__url+"/dialog?",PARAMS=PARAMS, method="GET", verbose=verbose)
        return response

    def hide(self, wid, verbose=False):
        """
        Hide and HTML browser in the Results Panel.

        :param wid: Window ID
        :param verbose: print more
        """

        PARAMS={"id":wid}

        response=api(url=self.__url+"/hide?",PARAMS=PARAMS, method="GET", verbose=verbose)
        return response

    def show(self, wid=None, text=None, title=None, url=None, verbose=False):
        """
        Launch an HTML browser in the Results Panel.

        :param wid: Window ID
        :param text: HTML text
        :param title: Window Title
        :param url: URL
        :param verbose: print more
        """

        PARAMS={}
        for p,v in zip(["id","text","title","url"],[wid,text,title,url]):
            if v:
                PARAMS[p]=v

        response=api(url=self.__url+"/show?",PARAMS=PARAMS, method="GET", verbose=verbose)
        return response
 
    def version(self, verbose=False):
        """
        Display the CyBrowser version.

        :param verbose: print more
        """
        response=api(url=self.__url+"/version",method="HTML", verbose=verbose)
        return response

    


from .base import *

class diffusion(object):
    """
    cytoscape session interface as shown in CyREST's swagger documentation for 'diffusion'.

    :param url: an url of the type 'http://' + host + ':' + str(port) + '/' + version + '/'.
    """

    def __init__(self, url):
        self.__url = url + 'commands/diffusion'

    def diffuse(self, verbose=False):
        """
        Diffusion will send the selected network view and its selected nodes to
        a web-based REST service to calculate network propagation. Results are
        returned and represented by columns in the node table.
        Columns are created for each execution of Diffusion and their names are
        returned in the response.
        The nodes you would like to use as input should be selected. This will
        be used to generate the contents of the diffusion_input column, which
        represents the query vector and corresponds to h in the diffusion equation.

        :param verbose: print more

        """
        response=api(url=self.__url+"/diffuse", method="POST", verbose=verbose)
        return response

    def diffuse_advanced(self, heatColumnName=None, time=None, verbose=False):
        """
        Diffusion will send the selected network view and its selected nodes to
        a web-based REST service to calculate network propagation. Results are
        returned and represented by columns in the node table.
        Columns are created for each execution of Diffusion and their names are
        returned in the response.

        :param heatColumnName (string, optional): A node column name intended
            to override the default table column 'diffusion_input'. This represents
            the query vector and corresponds to h in the diffusion equation. =
            ['HEKScore', 'JurkatScore', '(Use selected nodes)']
        :param time (string, optional): The extent of spread over the network.
            This corresponds to t in the diffusion equation.
        :param verbose: print more
        """
        PARAMS=set_param(["heatColumnName","time"],[heatColumnName,time])
        response=api(url=self.__url+"/diffuse_advanced", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response



from .base import *

class command(object):
    """
    cytoscape command as shown in CyREST's swagger documentation for 'Command'.

    :param url: an url of the type 'http://' + host + ':' + str(port) + '/' + version + '/'.
    """

    def __init__(self, url):
        self.__url = url + 'commands/command'

    def echo(self, variableName, verbose=False):
        """
        The echo command will display the value of the variable specified by the
        variableName argument, or all variables if variableName is not provided.

        :param variableName: The name of the variable or '*' to display the value of all variables.
        :param verbose: print more
        """
        PARAMS={"variableName":variableName}
        response=api(url=self.__url+"/echo", PARAMS=PARAMS, verbose=verbose)
        return response

    def open_dialog(self, verbose=False):
        """
        The command line dialog provides a field to enter commands and view
        results. It also provides the help command to display namespaces,
        commands, and arguments.

        :param verbose: print more
        """
        response=api(url=self.__url+"/open dialog", verbose=verbose)
        return response


    def pause(self, message=None, verbose=False):
        """
        The pause command displays a dialog with the text provided in the
        message argument and waits for the user to click OK

        :param message: a message to display. default=None
        :param verbose: print more
        """

        PARAMS=set_param(["message"],[message])
        response=api(url=self.__url+"/pause", PARAMS=PARAMS, verbose=verbose)
        return response

    
    def quit(self,verbose=False):
        """
        This command causes Cytoscape to exit. It is typically used at the end
        of a script file.

        :param verbose: print more
        """
        response=api(url=self.__url+"/quit", verbose=verbose)
        return response

    def run(self,script_file,args=None,verbose=False):
        """
        The run command will execute a command script from the file pointed to
        by the file argument, which should contain Cytoscape commands, one per
        line. Arguments to the script are provided by the args argument.

        :param script_file: file to run
        :param args: enter the script arguments as key:value pairs separated by
                        commas. eg. "arg1:value1,arg2:value2"
        :param verbose: print more
        """

        PARAMS=set_param(["file","args"],[script_file,args])
        response=api(url=self.__url+"/run", PARAMS=PARAMS, verbose=verbose)
        return response

    def sleep(self,duration,verbose=False):
        """
        The sleep command will pause processing for a period of time as specified
        by duration seconds. It is typically used as part of a command script.

        :param duration: enter the time in seconds to sleep
        :param verbose: print more
        """
        PARAMS={"duration":str(duration)}
        response=api(url=self.__url+"/sleep", PARAMS=PARAMS, verbose=verbose)
        return response

    


from .base import *

class vizmap(object):
    """
    cytoscape session interface as shown in CyREST's swagger documentation for 'vizmap'.

    :param url: an url of the type 'http://' + host + ':' + str(port) + '/' + version + '/'.
    """

    def __init__(self, url):
        self.__url = url + 'commands/vizmap'

    def apply(self, styles=None, verbose=False):
        """
        Applies the specified style to the selected views and returns the
        SUIDs of the affected views.

        :param styles (string): Name of Style to be applied to the selected
            views. = ['Directed', 'BioPAX_SIF', 'Bridging Reads Histogram:unique_0',
            'PSIMI 25 Style', 'Coverage Histogram:best&unique', 'Minimal',
            'Bridging Reads Histogram:best&unique_0', 'Coverage Histogram_0',
            'Big Labels', 'No Histogram:best&unique_0', 'Bridging Reads Histogram:best',
            'No Histogram_0', 'No Histogram:best&unique', 'Bridging Reads Histogram_0',
            'Ripple', 'Coverage Histogram:unique_0', 'Nested Network Style',
            'Coverage Histogram:best', 'Coverage Histogram:best&unique_0',
            'default black', 'No Histogram:best_0', 'No Histogram:unique',
            'No Histogram:unique_0', 'Solid', 'Bridging Reads Histogram:unique',
            'No Histogram:best', 'Coverage Histogram', 'BioPAX', 'Bridging Reads Histogram',
            'Coverage Histogram:best_0', 'Sample1', 'Universe', 'Bridging Reads Histogram:best_0',
            'Coverage Histogram:unique', 'Bridging Reads Histogram:best&unique',
            'No Histogram', 'default']
        :param verbose: print more
        :returns: SUIDs of the affected views

        """

        PARAMS=set_param(["styles"],[styles])
        response=api(url=self.__url+"/apply", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response


    def export(self, options=None, OutputFile=None, styles=None, verbose=False):
        """
        Exports the specified styles to a Cytoscape vizmap (XML) or a Cytoscape.js
        (JSON) file and returns the path to the saved file.

        :param options (string, optional): The format of the output file. =
            ['Style XML (*.xml)', 'Style for cytoscape.js (*.json)']
        :param OutputFile (string): The path name of the file where the styles
            must be saved to.
        :param styles (string, optional): The styles to be exported. If no styles
            are specified, only the current one is exported. = ['Big Labels',
            'BioPAX', 'BioPAX_SIF', 'Bridging Reads Histogram', 'Bridging Reads
            Histogram_0', 'Bridging Reads Histogram:best', 'Bridging Reads Histogram:best_0',
            'Bridging Reads Histogram:best&unique', 'Bridging Reads Histogram:best&unique_0',
            'Bridging Reads Histogram:unique', 'Bridging Reads Histogram:unique_0',
            'Coverage Histogram', 'Coverage Histogram_0', 'Coverage Histogram:best',
            'Coverage Histogram:best_0', 'Coverage Histogram:best&unique',
            'Coverage Histogram:best&unique_0', 'Coverage Histogram:unique', '
            Coverage Histogram:unique_0', 'default', 'default black', 'Directed',
            'Minimal', 'Nested Network Style', 'No Histogram', 'No Histogram_0',
            'No Histogram:best', 'No Histogram:best_0', 'No Histogram:best&unique',
            'No Histogram:best&unique_0', 'No Histogram:unique', 'No Histogram:unique_0',
            'PSIMI 25 Style', 'Ripple', 'Sample1', 'Solid', 'Universe']
        :param verbose: print more

        """

        PARAMS=set_param(["options","OutputFile", "styles"],[options,OutputFile,styles])
        response=api(url=self.__url+"/export", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response


    def load_file(self, style_xml, verbose=False):
        """
        Loads styles from a vizmap (XML or properties) file and returns the names of the loaded styles.

        :param afile (string): XML or properties file where one or more styles have been saved to.
        :param verbose: print more

        """
        PARAMS=set_param(["file"], [style_xml])
        response=api(url=self.__url+"/load file", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response

    def create_style(self,title=None,defaults=None,mappings=None,verbose=VERBOSE):
        """
        Creates a new visual style

        :param title: title of the visual style
        :param defaults: a list of dictionaries for each visualProperty
        :param mappings: a list of dictionaries for each visualProperty
        :param host: cytoscape host address, default=cytoscape_host
        :param port: cytoscape port, default=1234

        :returns: nothing
        """
        u=self.__url
        host=u.split("//")[1].split(":")[0]
        port=u.split(":")[2].split("/")[0]
        version=u.split(":")[2].split("/")[1]

        if defaults:
            defaults_=[]
            for d in defaults:
                if d:
                    defaults_.append(d)
            defaults=defaults_

        if mappings:
            mappings_=[]
            for m in mappings:
                if m:
                    mappings_.append(m)
            mappings=mappings_

        try:
            update_style(title=title,defaults=defaults,mappings=mappings,host=host,port=port)
            print("Existing style was updated.")
            sys.stdout.flush()
        except:
            print("Creating new style.")
            sys.stdout.flush()
            URL="http://"+str(host)+":"+str(port)+"/v1/styles"
            PARAMS={"title":title,\
                "defaults":defaults,\
                "mappings":mappings}
            r = requests.post(url = URL, json = PARAMS)
            checkresponse(r)

    def update_style(self, title=None,defaults=None,mappings=None, verbose=False):
        """
        Updates a visual style

        :param title: title of the visual style
        :param defaults: a list of dictionaries for each visualProperty
        :param mappings: a list of dictionaries for each visualProperty

        :returns: nothing
        """
        u=self.__url
        host=u.split("//")[1].split(":")[0]
        port=u.split(":")[2].split("/")[0]
        version=u.split(":")[2].split("/")[1]

        if defaults:
            defaults_=[]
            for d in defaults:
                if d:
                    defaults_.append(d)
            defaults=defaults_

        if mappings:
            mappings_=[]
            for m in mappings:
                if m:
                    mappings_.append(m)
            mappings=mappings_

        URL="http://"+str(host)+":"+str(port)+"/v1/styles/"+str(title)
        if verbose:
            print(URL)
            sys.stdout.flush()

        response = requests.get(URL).json()

        olddefaults=response["defaults"]
        oldmappings=response["mappings"]

        if mappings:
            mappings_visual_properties=[ m["visualProperty"] for m in mappings ]
            newmappings=[ m for m in oldmappings if m["visualProperty"] not in mappings_visual_properties ]
            for m in mappings:
                newmappings.append(m)
        else:
            newmappings=oldmappings

        if defaults:
            defaults_visual_properties=[ m["visualProperty"] for m in defaults ]
            newdefaults=[ m for m in olddefaults if m["visualProperty"] not in defaults_visual_properties ]
            for m in defaults:
                newdefaults.append(m)
        else:
            newdefaults=olddefaults

        r=requests.delete(URL)
        checkresponse(r)

        URL="http://"+str(host)+":"+str(port)+"/v1/styles"
        PARAMS={"title":title,\
            "defaults":newdefaults,\
            "mappings":newmappings}
        r = requests.post(url = URL, json = PARAMS)
        checkresponse(r)

    def mapVisualProperty(self, visualProperty=None,mappingType=None, mappingColumn=None,
                        lower=None,center=None,upper=None,\
                        discrete=None,\
                        network="current",table="node",\
                        namespace="default",verbose=False):
        """"
        Generates a dictionary for a given visual property

        :param visualProperty: visualProperty
        :param mappingType: mappingType
        :param mappingColumn: mappingColumn
        :param lower: for "continuous" mappings a list of the form [value,rgb_string]
        :param center: for "continuous" mappings a list of the form [value,rgb_string]
        :param upper: for "continuous" mappings a list of the form [value,rgb_string]
        :param discrete: for discrete mappings, a list of lists of the form [ list_of_keys, list_of_values ]
		:param network (string, optional): Specifies a network by name, or by
            SUID if the prefix SUID: is used. The keyword CURRENT, or a blank
            value can also be used to specify the current network.
        :param namespace (string, optional): Node, Edge, and Network objects support
            the default, local, and hidden namespaces. Root networks also support the
            shared namespace. Custom namespaces may be specified by Apps.
        :returns: a dictionary for the respective visual property
        """
        u=self.__url
        host=u.split("//")[1].split(":")[0]
        port=u.split(":")[2].split("/")[0]
        version=u.split(":")[2].split("/")[1]

        if type(network) != int:
            network=check_network(self,network,verbose=verbose)
            PARAMS=set_param(["columnList","namespace","network"],["SUID",namespace,network])
            networkID=api(namespace="network", command="get attribute",PARAMS=PARAMS, host=host,port=str(port),version=version)
            PARAMS=set_param(["columnList","namespace","network"],["name",namespace,network])
            networkname=api(namespace="network", command="get attribute",PARAMS=PARAMS, host=host,port=str(port),version=version)
            network=networkID[0]["SUID"]
            networkname=networkname[0]["name"]

        URL="http://"+str(host)+":"+str(port)+"/v1/networks/"+str(network)+"/tables/"+namespace+table+"/columns/"
        if verbose:
            print(URL)
            sys.stdout.flush()
        response = requests.get(URL).json()

        mappingColumnType=None
        for r in response:
            if r["name"]==mappingColumn:
                mappingColumnType=r["type"]
                break
        if not mappingColumnType:
            print("For mappingType: "+mappingType+" it was not possible to find a  mappingColumnType.")
            sys.stdout.flush()

        PARAMS={"mappingType" : mappingType,\
                "mappingColumn" : mappingColumn,
                "mappingColumnType" : mappingColumnType,
                "visualProperty" : visualProperty}
        if mappingType == "continuous":
            PARAMS["points"]=[{"value" : lower[0],\
                            "lesser" : lower[1],\
                            "equal" : lower[1],\
                            "greater" : lower[1]},\
                            {"value" : center[0],
                            "lesser" : center[1],
                            "equal" : center[1],
                            "greater" : center[1] },\
                            {"value" : upper[0],\
                            "lesser" : upper[1],\
                            "equal" : upper[1],\
                            "greater" : upper[1]}]

        if discrete:
            PARAMS["map"]=[]
            for k,v in zip(discrete[0],discrete[1]):
                PARAMS["map"].append({ "key":k,"value":v})

        if not mappingColumnType:
            res=None
        else:
            res=PARAMS

        return res

    def simple_defaults(self, defaults_dic):
        """
        Simplifies defaults.

        :param defaults_dic: a dictionary of the form { visualProperty_A:value_A, visualProperty_B:value_B, ..}

        :returns: a list of dictionaries with each item corresponding to a given key in defaults_dic
        """

        defaults=[]
        for d in defaults_dic.keys():
            dic={}
            dic["visualProperty"]=d
            dic["value"]=defaults_dic[d]
            defaults.append(dic)
        return defaults



from .base import *

class layout(object):
	"""
	cytoscape session interface as shown in CyREST's swagger documentation for 'layout'.

	:param url: an url of the type 'http://' + host + ':' + str(port) + '/' + version + '/'.
	"""

	def __init__(self, url):
		self.__url = url + 'commands/layout'

	def apply_preferred(self, network=None, verbose=False):
		"""
		Executes the current preferred layout. Default is grid.

		:param networkSelected (string, optional): Specifies a network by name,
			or by SUID if the prefix SUID: is used. The keyword CURRENT, or a
			blank value can also be used to specify the current network.
		:param verbose: print more

		"""
		network=check_network(self,network,verbose=verbose)
		PARAMS=set_param(["network"],[network])
		response=api(url=self.__url+"/apply preferred", PARAMS=PARAMS, method="POST", verbose=verbose)
		return response

    
	def attribute_circle(self, EdgeAttribute=None, network=None, \
	NodeAttribute=None, nodeList=None, singlePartition=None,\
	spacing=None, verbose=False):
		"""
		Execute the Attribute Circle Layout on a network.

		:param EdgeAttribute (string, optional): The name of the edge column
			containing numeric values that will be used as weights in the layout
			algorithm. Only columns containing numeric values are shown
		:param network (string, optional): Specifies a network by name, or by
			SUID if the prefix SUID: is used. The keyword CURRENT, or a blank
			value can also be used to specify the current network.
		:param NodeAttribute (string, optional): The name of the node column
			containing numeric values that will be used as weights in the layout
			algorithm. Only columns containing numeric values are shown
		:param nodeList (string, optional): Specifies a list of nodes. The
			keywords all, selected, or unselected can be used to specify nodes
			by their selection state. The pattern COLUMN:VALUE sets this parameter
			to any rows that contain the specified column value; if the COLUMN
			prefix is not used, the NAME column is matched by default. A list of
			COLUMN:VALUE pairs of the format COLUMN1:VALUE1,COLUMN2:VALUE2,...
			can be used to match multiple values.
		:param singlePartition (string, optional): Don't partition graph before
			layout, only boolean values allowed: true or false
		:param spacing (string, optional): Circle size, in numeric value
		:param verbose: print more

		"""
		network=check_network(self,network,verbose=verbose)
		PARAMS=set_param(["EdgeAttribute","network","NodeAttribute","nodeList","singlePartition","spacing"],\
		[EdgeAttribute,network,NodeAttribute,nodeList,singlePartition,spacing])
		response=api(url=self.__url+"/attribute-circle", PARAMS=PARAMS, method="POST", verbose=verbose)
		return response

    
	def attributes_layout(self, EdgeAttribute=None, maxwidth=None, minrad=None, \
	network=None, NodeAttribute=None,nodeList=None, radmult=None, \
	spacingx=None, spacingy=None, verbose=False):
		"""
		Execute the Group Attributes Layout on a network

		:param EdgeAttribute (string, optional): The name of the edge column
			containing numeric values that will be used as weights in the layout
				algorithm. Only columns containing numeric values are shown
		:param maxwidth (string, optional): Maximum width of a row, in numeric value
		:param minrad (string, optional): Minimum width of a partition, in
			numeric value
		:param network (string, optional): Specifies a network by name, or by
			SUID if the prefix SUID: is used. The keyword CURRENT, or a blank
			value can also be used to specify the current network.
		:param NodeAttribute (string, optional): The name of the node column
			containing numeric values that will be used as weights in the layout
			algorithm. Only columns containing numeric values are shown
		:param nodeList (string, optional): Specifies a list of nodes. The
			keywords all, selected, or unselected can be used to specify nodes
			by their selection state. The pattern COLUMN:VALUE sets this parameter
			to any rows that contain the specified column value; if the COLUMN
			prefix is not used, the NAME column is matched by default. A list
			of COLUMN:VALUE pairs of the format COLUMN1:VALUE1,COLUMN2:VALUE2,...
			can be used to match multiple values.
		:param radmult (string, optional): Minimum width of a partition, in
			numeric value
		:param spacingx (string, optional): Horizontal spacing between two
			partitions in a row, in numeric value
		:param spacingy (string, optional): Vertical spacing between the largest
			partitions of two rows, in numeric value
		:param verbose: print more

		"""
		network=check_network(self,network,verbose=verbose)
		PARAMS=set_param(["EdgeAttribute","network","NodeAttribute","nodeList","singlePartition","spacing"],\
		[EdgeAttribute, maxwidth, \
		minrad, network, NodeAttribute,nodeList, radmult, \
		spacingx, spacingy])
		response=api(url=self.__url+"/attributes-layout", PARAMS=PARAMS, method="POST", verbose=verbose)
		return response

	def circular(self,EdgeAttribute=None,leftEdge=None,network=None,\
	NodeAttribute=None,nodeHorizontalSpacing=None,nodeList=None,\
	nodeVerticalSpacing=None,rightMargin=None,singlePartition=None,topEdge=None,\
	verbose=None):
		"""
		Execute the Circular Layout on a network

		:param EdgeAttribute (string, optional): The name of the edge column contai
			ning numeric values that will be used as weights in the layout algor
			ithm. Only columns containing numeric values are shown
		:param leftEdge (string, optional): Left edge margin, in numeric value
		:param network (string, optional): Specifies a network by name, or by SUID
			if the prefix SUID: is used. The keyword CURRENT, or a blank value c
			an also be used to specify the current network.
		:param NodeAttribute (string, optional): The name of the node column contai
			ning numeric values that will be used as weights in the layout algor
			ithm. Only columns containing numeric values are shown
		:param nodeHorizontalSpacing (string, optional): Horizontal spacing between
				nodes, in numeric value
		:param nodeList (string, optional): Specifies a list of nodes. The keywords
				all, selected, or unselected can be used to specify nodes by their
			selection state. The pattern COLUMN:VALUE sets this parameter to any
				rows that contain the specified column value; if the COLUMN prefix
			is not used, the NAME column is matched by default. A list of COLUMN
			:VALUE pairs of the format COLUMN1:VALUE1,COLUMN2:VALUE2,... can be
			used to match multiple values.
		:param nodeVerticalSpacing (string, optional): Vertical spacing between nod
			es, in numeric value
		:param rightMargin (string, optional): Right edge margin, in numeric value
		:param singlePartition (string, optional): Don't partition graph before lay
			out; only boolean values are allowed: true or false
		:param topEdge (string, optional): Top edge margin, in numeric value
		"""
		network=check_network(self,network,verbose=verbose)
		PARAMS=set_param(['EdgeAttribute','leftEdge','network','NodeAttribute',\
		'nodeHorizontalSpacing','nodeList','nodeVerticalSpacing','rightMargin',\
		'singlePartition','topEdge'],[EdgeAttribute,leftEdge,network,NodeAttribute,\
		nodeHorizontalSpacing,nodeList,nodeVerticalSpacing,rightMargin,\
		singlePartition,topEdge])
		response=api(url=self.__url+"/circular", PARAMS=PARAMS, method="POST", verbose=verbose)
		return response

	def copycat(self,gridUnmapped=None,selectUnmapped=None,sourceColumn=None,\
	sourceNetwork=None,targetColumn=None,targetNetwork=None,verbose=None):
		"""
		Sets the coordinates for each node in the target network to the coordinates
		of a matching node in the source network.
		Optional parameters such as gridUnmapped and selectUnmapped determine
		the behavior of target network nodes that could not be matched.

		:param gridUnmapped (string, optional): If this is set to true, any nodes i
			n the target network that could not be matched to a node in the sour
			ce network will be laid out in a grid
		:param selectUnmapped (string, optional): If this is set to true, any nodes
				in the target network that could not be matched to a node in the so
			urce network will be selected in the target network
		:param sourceColumn (string): The name of column in the node table used to
			match nodes
		:param sourceNetwork (string): The name of network to get node coordinates
			from
		:param targetColumn (string): The name of column in the node table used to
			match nodes
		:param targetNetwork (string): The name of the network to apply coordinates
				to.
		"""
		PARAMS=set_param(['gridUnmapped','selectUnmapped','sourceColumn',\
		'sourceNetwork','targetColumn','targetNetwork'],[gridUnmapped,\
		selectUnmapped,sourceColumn,sourceNetwork,targetColumn,targetNetwork])
		response=api(url=self.__url+"/copycat", PARAMS=PARAMS, method="POST", verbose=verbose)
		return response

	def cose(self,compoundGravityRange=None,compoundGravityStrength=None,\
		EdgeAttribute=None,gravityRange=None,gravityStrength=None,idealEdgeLength=None,\
		incremental=None,LayoutQuality=None,network=None,NodeAttribute=None,\
		nodeList=None,repulsionStrength=None,smartEdgeLengthCalc=None,\
		smartRepulsionRangeCalc=None,springStrength=None,verbose=None):
		"""
		Execute the Compound Spring Embedder (CoSE) on a network

		:param compoundGravityRange (string, optional): Compound gravity range (0-1
			00)
		:param compoundGravityStrength (string, optional): Compound gravity strengt
			h (0-100)
		:param EdgeAttribute (string, optional): The name of the edge column contai
			ning numeric values that will be used as weights in the layout algor
			ithm. Only columns containing numeric values are shown
		:param gravityRange (string, optional): Gravity range (0-100)
		:param gravityStrength (string, optional): Gravity strength (0-100)
		:param idealEdgeLength (string, optional): Ideal edge length, any positive
			integer
		:param incremental (string, optional): Incremental; whether the algorithm w
			ill be applied incrementally; boolean values only, true or false; de
			faults to false
		:param LayoutQuality (string, optional): Layout quality; allowed values are
				Proof, Default and Draft = ['Proof', 'Default', 'Draft']
		:param network (string, optional): Specifies a network by name, or by SUID
			if the prefix SUID: is used. The keyword CURRENT, or a blank value c
			an also be used to specify the current network.
		:param NodeAttribute (string, optional): The name of the node column contai
			ning numeric values that will be used as weights in the layout algor
			ithm. Only columns containing numeric values are shown
		:param nodeList (string, optional): Specifies a list of nodes. The keywords
				all, selected, or unselected can be used to specify nodes by their
			selection state. The pattern COLUMN:VALUE sets this parameter to any
				rows that contain the specified column value; if the COLUMN prefix
			is not used, the NAME column is matched by default. A list of COLUMN
			:VALUE pairs of the format COLUMN1:VALUE1,COLUMN2:VALUE2,... can be
			used to match multiple values.
		:param repulsionStrength (string, optional): Repulsion strength (0-100)
		:param smartEdgeLengthCalc (string, optional): Use smart edge length calcul
			ation; boolean values only, true or false; defaults to true
		:param smartRepulsionRangeCalc (string, optional): Use smart repulsion rang
			e calculation; boolean values only, true or false; defaults to true
		:param springStrength (string, optional): Spring strength (0-100)
		"""
		network=check_network(self,network,verbose=verbose)
		PARAMS=set_param(['compoundGravityRange','compoundGravityStrength',\
		'EdgeAttribute','gravityRange','gravityStrength','idealEdgeLength',\
		'incremental','LayoutQuality','network','NodeAttribute','nodeList',\
		'repulsionStrength','smartEdgeLengthCalc','smartRepulsionRangeCalc',\
		'springStrength'],[compoundGravityRange,compoundGravityStrength,EdgeAttribute,\
		gravityRange,gravityStrength,idealEdgeLength,incremental,LayoutQuality,\
		network,NodeAttribute,nodeList,repulsionStrength,smartEdgeLengthCalc,\
		smartRepulsionRangeCalc,springStrength])
		response=api(url=self.__url+"/cose", PARAMS=PARAMS, method="POST", verbose=verbose)
		return response

	def degree_circle(self,EdgeAttribute=None,network=None,NodeAttribute=None,\
		nodeList=None,singlePartition=None,verbose=None):
		"""
		Execute the Degree Sorted Circle Layout on a network.

		:param EdgeAttribute (string, optional): The name of the edge column contai
			ning numeric values that will be used as weights in the layout algor
			ithm. Only columns containing numeric values are shown
		:param network (string, optional): Specifies a network by name, or by SUID
			if the prefix SUID: is used. The keyword CURRENT, or a blank value c
			an also be used to specify the current network.
		:param NodeAttribute (string, optional): The name of the node column contai
			ning numeric values that will be used as weights in the layout algor
			ithm. Only columns containing numeric values are shown
		:param nodeList (string, optional): Specifies a list of nodes. The keywords
				all, selected, or unselected can be used to specify nodes by their
			selection state. The pattern COLUMN:VALUE sets this parameter to any
				rows that contain the specified column value; if the COLUMN prefix
			is not used, the NAME column is matched by default. A list of COLUMN
			:VALUE pairs of the format COLUMN1:VALUE1,COLUMN2:VALUE2,... can be
			used to match multiple values.
		:param singlePartition (string, optional): Don't partition graph before lay
			out; boolean values only, true or false; defaults to false
		"""
		network=check_network(self,network,verbose=verbose)
		PARAMS=set_param(['EdgeAttribute','network','NodeAttribute','nodeList',\
		'singlePartition'],[EdgeAttribute,network,NodeAttribute,nodeList,\
		singlePartition])
		response=api(url=self.__url+"/degree-circle", PARAMS=PARAMS, method="POST", verbose=verbose)
		return response

    
	def force_directed(self,defaultEdgeWeight=None,defaultNodeMass=None,\
		defaultSpringCoefficient=None,defaultSpringLength=None,EdgeAttribute=None,\
		isDeterministic=None,maxWeightCutoff=None,minWeightCutoff=None,network=None,\
		NodeAttribute=None,nodeList=None,numIterations=None,singlePartition=None,\
		Type=None,verbose=None):
		"""
		Execute the Prefuse Force Directed Layout on a network

		:param defaultEdgeWeight (string, optional): The default edge weight to con
			sider, default is 0.5
		:param defaultNodeMass (string, optional): Default Node Mass, in numeric va
			lue
		:param defaultSpringCoefficient (string, optional): Default Spring Coeffici
			ent, in numeric value
		:param defaultSpringLength (string, optional): Default Spring Length, in nu
			meric value
		:param EdgeAttribute (string, optional): The name of the edge column contai
			ning numeric values that will be used as weights in the layout algor
			ithm. Only columns containing numeric values are shown
		:param isDeterministic (string, optional): Force deterministic layouts (slo
			wer); boolean values only, true or false; defaults to false
		:param maxWeightCutoff (string, optional): The maximum edge weight to consi
			der, default to the Double.MAX value
		:param minWeightCutoff (string, optional): The minimum edge weight to consi
			der, numeric values, default is 0
		:param network (string, optional): Specifies a network by name, or by SUID
			if the prefix SUID: is used. The keyword CURRENT, or a blank value c
			an also be used to specify the current network.
		:param NodeAttribute (string, optional): The name of the node column contai
			ning numeric values that will be used as weights in the layout algor
			ithm. Only columns containing numeric values are shown
		:param nodeList (string, optional): Specifies a list of nodes. The keywords
				all, selected, or unselected can be used to specify nodes by their
			selection state. The pattern COLUMN:VALUE sets this parameter to any
				rows that contain the specified column value; if the COLUMN prefix
			is not used, the NAME column is matched by default. A list of COLUMN
			:VALUE pairs of the format COLUMN1:VALUE1,COLUMN2:VALUE2,... can be
			used to match multiple values.
		:param numIterations (string, optional): Number of Iterations, in numeric v
			alue
		:param singlePartition (string, optional): Don't partition graph before lay
			out; boolean values only, true or false; defaults to false
		:param Type (string, optional): How to interpret weight values; must be one
				of Heuristic, -Log(value), 1 - normalized value and normalized valu
			e. Defaults to Heuristic = ['Heuristic', '-Log(value)', '1 - normali
			zed value', 'normalized value']
		"""
		network=check_network(self,network,verbose=verbose)
		PARAMS=set_param(['defaultEdgeWeight','defaultNodeMass','defaultSpringCoefficient',\
		'defaultSpringLength','EdgeAttribute','isDeterministic','maxWeightCutoff',\
		'minWeightCutoff','network','NodeAttribute','nodeList','numIterations',\
		'singlePartition','Type'],[defaultEdgeWeight,defaultNodeMass,\
		defaultSpringCoefficient,defaultSpringLength,EdgeAttribute,isDeterministic,\
		maxWeightCutoff,minWeightCutoff,network,NodeAttribute,nodeList,numIterations,\
		singlePartition,Type])
		response=api(url=self.__url+"/force-directed", PARAMS=PARAMS, method="POST", verbose=verbose)
		return response

	def force_directed_cl(self,defaultEdgeWeight=None,defaultNodeMass=None,\
		defaultSpringCoefficient=None,defaultSpringLength=None,EdgeAttribute=None,\
		fromScratch=None,isDeterministic=None,maxWeightCutoff=None,minWeightCutoff=None,\
		network=None,NodeAttribute=None,nodeList=None,numIterations=None,\
		numIterationsEdgeRepulsive=None,singlePartition=None,Type=None,verbose=None):
		"""
		Execute the Prefuse Force Directed OpenCL Layout on a network.

		:param defaultEdgeWeight (string, optional): The default edge weight to con
			sider, default is 0.5
		:param defaultNodeMass (string, optional):
		:param defaultSpringCoefficient (string, optional):
		:param defaultSpringLength (string, optional):
		:param EdgeAttribute (string, optional): The name of the edge column contai
			ning numeric values that will be used as weights in the layout algor
			ithm. Only columns containing numeric values are shown
		:param fromScratch (string, optional):
		:param isDeterministic (string, optional):
		:param maxWeightCutoff (string, optional): The maximum edge weight to consi
			der, default to the Double.MAX value
		:param minWeightCutoff (string, optional): The minimum edge weight to consi
			der, numeric values, default is 0
		:param network (string, optional): Specifies a network by name, or by SUID
			if the prefix SUID: is used. The keyword CURRENT, or a blank value c
			an also be used to specify the current network.
		:param NodeAttribute (string, optional): The name of the node column contai
			ning numeric values that will be used as weights in the layout algor
			ithm. Only columns containing numeric values are shown
		:param nodeList (string, optional): Specifies a list of nodes. The keywords
				all, selected, or unselected can be used to specify nodes by their
			selection state. The pattern COLUMN:VALUE sets this parameter to any
				rows that contain the specified column value; if the COLUMN prefix
			is not used, the NAME column is matched by default. A list of COLUMN
			:VALUE pairs of the format COLUMN1:VALUE1,COLUMN2:VALUE2,... can be
			used to match multiple values.
		:param numIterations (string, optional):
		:param numIterationsEdgeRepulsive (string, optional):
		:param singlePartition (string, optional):
		:param Type (string, optional): How to interpret weight values; must be one
				of Heuristic, -Log(value), 1 - normalized value and normalized valu
			e. Defaults to Heuristic = ['Heuristic', '-Log(value)', '1 - normali
			zed value', 'normalized value']
		"""
		network=check_network(self,network,verbose=verbose)
		PARAMS=set_param(['defaultEdgeWeight','defaultNodeMass',\
		'defaultSpringCoefficient','defaultSpringLength','EdgeAttribute',\
		'fromScratch','isDeterministic','maxWeightCutoff','minWeightCutoff',\
		'network','NodeAttribute','nodeList','numIterations',\
		'numIterationsEdgeRepulsive','singlePartition','Type'],\
		[defaultEdgeWeight,defaultNodeMass,defaultSpringCoefficient,\
		defaultSpringLength,EdgeAttribute,fromScratch,isDeterministic,\
		maxWeightCutoff,minWeightCutoff,network,NodeAttribute,nodeList,\
		numIterations,numIterationsEdgeRepulsive,singlePartition,Type])
		response=api(url=self.__url+"/force-directed-cl", PARAMS=PARAMS, method="POST", verbose=verbose)
		return response


	def fruchterman_rheingold(self,attraction_multiplier=None,conflict_avoidance=None,\
		defaultEdgeWeight=None,EdgeAttribute=None,gravity_multiplier=None,\
		layout3D=None,max_distance_factor=None,maxWeightCutoff=None,minWeightCutoff=None,\
		network=None,nIterations=None,NodeAttribute=None,nodeList=None,randomize=None,\
		repulsion_multiplier=None,singlePartition=None,spread_factor=None,\
		temperature=None,Type=None,update_iterations=None,verbose=None):
		"""
		Execute the Edge-weighted Force directed (BioLayout) on a network

		:param attraction_multiplier (string, optional): Divisor to calculate the a
			ttraction force, in numeric value
		:param conflict_avoidance (string, optional): Constant force applied to avo
			id conflicts, in numeric value
		:param defaultEdgeWeight (string, optional): The default edge weight to con
			sider, default is 0.5
		:param EdgeAttribute (string, optional): The name of the edge column contai
			ning numeric values that will be used as weights in the layout algor
			ithm. Only columns containing numeric values are shown
		:param gravity_multiplier (string, optional): Multiplier to calculate the g
			ravity force, in numeric value
		:param layout3D (string, optional): Layout nodes in 3D; boolean values only
			, true or false; defaults to true
		:param max_distance_factor (string, optional): Percent of graph used for no
			de repulsion calculations, in numeric value
		:param maxWeightCutoff (string, optional): The maximum edge weight to consi
			der, default to the Double.MAX value
		:param minWeightCutoff (string, optional): The minimum edge weight to consi
			der, numeric values, default is 0
		:param network (string, optional): Specifies a network by name, or by SUID
			if the prefix SUID: is used. The keyword CURRENT, or a blank value c
			an also be used to specify the current network.
		:param nIterations (string, optional): Number of iterations, in numeric val
			ue
		:param NodeAttribute (string, optional): The name of the node column contai
			ning numeric values that will be used as weights in the layout algor
			ithm. Only columns containing numeric values are shown
		:param nodeList (string, optional): Specifies a list of nodes. The keywords
				all, selected, or unselected can be used to specify nodes by their
			selection state. The pattern COLUMN:VALUE sets this parameter to any
				rows that contain the specified column value; if the COLUMN prefix
			is not used, the NAME column is matched by default. A list of COLUMN
			:VALUE pairs of the format COLUMN1:VALUE1,COLUMN2:VALUE2,... can be
			used to match multiple values.
		:param randomize (string, optional): Randomize graph before layout; boolean
				values only, true or false; defaults to true
		:param repulsion_multiplier (string, optional): Multiplier to calculate the
				repulsion force, in numeric value
		:param singlePartition (string, optional): Don't partition graph before lay
			out; boolean values only, true or false; defaults to false
		:param spread_factor (string, optional): Amount of extra room for layout, i
			n numeric value
		:param temperature (string, optional): Initial temperature, in numeric valu
			e
		:param Type (string, optional): How to interpret weight values; must be one
				of Heuristic, -Log(value), 1 - normalized value and normalized valu
			e. Defaults to Heuristic = ['Heuristic', '-Log(value)', '1 - normali
			zed value', 'normalized value']
		:param update_iterations (string, optional): Number of iterations before up
			dating display, in numeric value (0: update only at end)
		"""
		network=check_network(self,network,verbose=verbose)
		PARAMS=set_param(['attraction_multiplier','conflict_avoidance',\
		'defaultEdgeWeight','EdgeAttribute','gravity_multiplier','layout3D',\
		'max_distance_factor','maxWeightCutoff','minWeightCutoff','network',\
		'nIterations','NodeAttribute','nodeList','randomize','repulsion_multiplier',\
		'singlePartition','spread_factor','temperature','Type','update_iterations'],\
		[attraction_multiplier,conflict_avoidance,defaultEdgeWeight,EdgeAttribute,\
		gravity_multiplier,layout3D,max_distance_factor,maxWeightCutoff,\
		minWeightCutoff,network,nIterations,NodeAttribute,nodeList,randomize,\
		repulsion_multiplier,singlePartition,spread_factor,temperature,Type,\
		update_iterations])
		response=api(url=self.__url+"/fruchterman-rheingold", PARAMS=PARAMS, method="POST", verbose=verbose)
		return response


	def genemania_force_directed(self,curveSteepness=None,defaultEdgeWeight=None,\
		defaultSpringCoefficient=None,defaultSpringLength=None,EdgeAttribute=None,\
		ignoreHiddenElements=None,isDeterministic=None,maxNodeMass=None,\
		maxWeightCutoff=None,midpointEdges=None,minNodeMass=None,minWeightCutoff=None,\
		network=None,NodeAttribute=None,nodeList=None,numIterations=None,\
		singlePartition=None,Type=None,verbose=None):
		"""
		Execute the GeneMANIA Force Directed Layout on a network.

		:param curveSteepness (string, optional):
		:param defaultEdgeWeight (string, optional): The default edge weight to con
			sider, default is 0.5
		:param defaultSpringCoefficient (string, optional):
		:param defaultSpringLength (string, optional):
		:param EdgeAttribute (string, optional): The name of the edge column contai
			ning numeric values that will be used as weights in the layout algor
			ithm. Only columns containing numeric values are shown
		:param ignoreHiddenElements (string, optional):
		:param isDeterministic (string, optional):
		:param maxNodeMass (string, optional):
		:param maxWeightCutoff (string, optional): The maximum edge weight to consi
			der, default to the Double.MAX value
		:param midpointEdges (string, optional):
		:param minNodeMass (string, optional):
		:param minWeightCutoff (string, optional): The minimum edge weight to consi
			der, numeric values, default is 0
		:param network (string, optional): Specifies a network by name, or by SUID
			if the prefix SUID: is used. The keyword CURRENT, or a blank value c
			an also be used to specify the current network.
		:param NodeAttribute (string, optional): The name of the node column contai
			ning numeric values that will be used as weights in the layout algor
			ithm. Only columns containing numeric values are shown
		:param nodeList (string, optional): Specifies a list of nodes. The keywords
				all, selected, or unselected can be used to specify nodes by their
			selection state. The pattern COLUMN:VALUE sets this parameter to any
				rows that contain the specified column value; if the COLUMN prefix
			is not used, the NAME column is matched by default. A list of COLUMN
			:VALUE pairs of the format COLUMN1:VALUE1,COLUMN2:VALUE2,... can be
			used to match multiple values.
		:param numIterations (string, optional):
		:param singlePartition (string, optional):
		:param Type (string, optional): How to interpret weight values; must be one
				of Heuristic, -Log(value), 1 - normalized value and normalized valu
			e. Defaults to Heuristic = ['Heuristic', '-Log(value)', '1 - normali
			zed value', 'normalized value']
		"""
		network=check_network(self,network,verbose=verbose)
		PARAMS=set_param(['curveSteepness','defaultEdgeWeight',\
		'defaultSpringCoefficient','defaultSpringLength','EdgeAttribute',\
		'ignoreHiddenElements','isDeterministic','maxNodeMass','maxWeightCutoff',\
		'midpointEdges','minNodeMass','minWeightCutoff','network','NodeAttribute',\
		'nodeList','numIterations','singlePartition','Type'],[curveSteepness,\
		defaultEdgeWeight,defaultSpringCoefficient,defaultSpringLength,EdgeAttribute,\
		ignoreHiddenElements,isDeterministic,maxNodeMass,maxWeightCutoff,\
		midpointEdges,minNodeMass,minWeightCutoff,network,NodeAttribute,nodeList,\
		numIterations,singlePartition,Type])
		response=api(url=self.__url+"/genemania-force-directed", PARAMS=PARAMS, method="POST", verbose=verbose)
		return response
    

	def get_preferred(self,network=None,verbose=None):
		"""
		Returns the name of the current preferred layout or empty string if not
		set. Default is grid.

		:param network (string, optional): Gets the name of the current preferred l
			ayout
		"""
		network=check_network(self,network,verbose=verbose)
		PARAMS=set_param(['network'],[network])
		response=api(url=self.__url+"/get preferred", PARAMS=PARAMS, method="POST", verbose=verbose)
		return response

	def grid(self,EdgeAttribute=None,network=None,NodeAttribute=None,\
		nodeHorizontalSpacing=None,nodeList=None,nodeVerticalSpacing=None,verbose=None):
		"""
		Execute the Grid Layout on a network.

		:param EdgeAttribute (string, optional): The name of the edge column contai
			ning numeric values that will be used as weights in the layout algor
			ithm. Only columns containing numeric values are shown
		:param network (string, optional): Specifies a network by name, or by SUID
			if the prefix SUID: is used. The keyword CURRENT, or a blank value c
			an also be used to specify the current network.
		:param NodeAttribute (string, optional): The name of the node column contai
			ning numeric values that will be used as weights in the layout algor
			ithm. Only columns containing numeric values are shown
		:param nodeHorizontalSpacing (string, optional): Horizontal spacing between
				nodes, in numeric value
		:param nodeList (string, optional): Specifies a list of nodes. The keywords
				all, selected, or unselected can be used to specify nodes by their
			selection state. The pattern COLUMN:VALUE sets this parameter to any
				rows that contain the specified column value; if the COLUMN prefix
			is not used, the NAME column is matched by default. A list of COLUMN
			:VALUE pairs of the format COLUMN1:VALUE1,COLUMN2:VALUE2,... can be
			used to match multiple values.
		:param nodeVerticalSpacing (string, optional): Vertical spacing between nod
			es, in numeric value
		"""
		network=check_network(self,network,verbose=verbose)
		PARAMS=set_param(['EdgeAttribute','network','NodeAttribute',\
		'nodeHorizontalSpacing','nodeList','nodeVerticalSpacing'],\
		[EdgeAttribute,network,NodeAttribute,nodeHorizontalSpacing,nodeList,\
		nodeVerticalSpacing])
		response=api(url=self.__url+"/grid", PARAMS=PARAMS, method="POST", verbose=verbose)
		return response


	def hierarchical(self,bandGap=None,componentSpacing=None,EdgeAttribute=None,\
		leftEdge=None,network=None,NodeAttribute=None,nodeHorizontalSpacing=None,\
		nodeList=None,nodeVerticalSpacing=None,rightMargin=None,topEdge=None,\
		verbose=None):
		"""
		Execute the Hierarchical Layout on a network.

		:param bandGap (string, optional): Band gap, in numeric value
		:param componentSpacing (string, optional): Component spacing, in numeric v
			alue
		:param EdgeAttribute (string, optional): The name of the edge column contai
			ning numeric values that will be used as weights in the layout algor
			ithm. Only columns containing numeric values are shown
		:param leftEdge (string, optional): Left edge margin, in numeric value
		:param network (string, optional): Specifies a network by name, or by SUID
			if the prefix SUID: is used. The keyword CURRENT, or a blank value c
			an also be used to specify the current network.
		:param NodeAttribute (string, optional): The name of the node column contai
			ning numeric values that will be used as weights in the layout algor
			ithm. Only columns containing numeric values are shown
		:param nodeHorizontalSpacing (string, optional): Horizontal spacing between
				nodes, in numeric value
		:param nodeList (string, optional): Specifies a list of nodes. The keywords
				all, selected, or unselected can be used to specify nodes by their
			selection state. The pattern COLUMN:VALUE sets this parameter to any
				rows that contain the specified column value; if the COLUMN prefix
			is not used, the NAME column is matched by default. A list of COLUMN
			:VALUE pairs of the format COLUMN1:VALUE1,COLUMN2:VALUE2,... can be
			used to match multiple values.
		:param nodeVerticalSpacing (string, optional): Vertical spacing between nod
			es, in numeric value
		:param rightMargin (string, optional): Right edge margin, in numeric value
		:param topEdge (string, optional): Top edge margin, in numeric value
		"""
		network=check_network(self,network,verbose=verbose)
		PARAMS=set_param(['bandGap','componentSpacing','EdgeAttribute','leftEdge',\
		'network','NodeAttribute','nodeHorizontalSpacing','nodeList',\
		'nodeVerticalSpacing','rightMargin','topEdge'],[bandGap,componentSpacing,\
		EdgeAttribute,leftEdge,network,NodeAttribute,nodeHorizontalSpacing,\
		nodeList,nodeVerticalSpacing,rightMargin,topEdge])
		response=api(url=self.__url+"/hierarchical", PARAMS=PARAMS, method="POST", verbose=verbose)
		return response


	def isom(self,coolingFactor=None,EdgeAttribute=None,initialAdaptation=None,\
		maxEpoch=None,minAdaptation=None,minRadius=None,network=None,NodeAttribute=None,\
		nodeList=None,radius=None,radiusConstantTime=None,singlePartition=None,\
		sizeFactor=None,verbose=None):
		"""
		Execute the Inverted Self-Organizing Map Layout on a network.

		:param coolingFactor (string, optional): Cooling factor, in numeric value
		:param EdgeAttribute (string, optional): The name of the edge column contai
			ning numeric values that will be used as weights in the layout algor
			ithm. Only columns containing numeric values are shown
		:param initialAdaptation (string, optional): Initial adaptation, in numeric
				value
		:param maxEpoch (string, optional): Number of iterations, in numeric value
		:param minAdaptation (string, optional): Minimum adaptation value, in numer
			ic value
		:param minRadius (string, optional): Minimum radius, in numeric value
		:param network (string, optional): Specifies a network by name, or by SUID
			if the prefix SUID: is used. The keyword CURRENT, or a blank value c
			an also be used to specify the current network.
		:param NodeAttribute (string, optional): The name of the node column contai
			ning numeric values that will be used as weights in the layout algor
			ithm. Only columns containing numeric values are shown
		:param nodeList (string, optional): Specifies a list of nodes. The keywords
				all, selected, or unselected can be used to specify nodes by their
			selection state. The pattern COLUMN:VALUE sets this parameter to any
				rows that contain the specified column value; if the COLUMN prefix
			is not used, the NAME column is matched by default. A list of COLUMN
			:VALUE pairs of the format COLUMN1:VALUE1,COLUMN2:VALUE2,... can be
			used to match multiple values.
		:param radius (string, optional): Radius, in numeric value
		:param radiusConstantTime (string, optional): Radius constant, in numeric v
			alue
		:param singlePartition (string, optional): Don't partition graph before lay
			out; boolean values only, true or false; defaults to false
		:param sizeFactor (string, optional): Size factor, in numeric value
		"""
		network=check_network(self,network,verbose=verbose)
		PARAMS=set_param(['coolingFactor','EdgeAttribute','initialAdaptation',\
		'maxEpoch','minAdaptation','minRadius','network','NodeAttribute','nodeList',\
		'radius','radiusConstantTime','singlePartition','sizeFactor'],[coolingFactor,\
		EdgeAttribute,initialAdaptation,maxEpoch,minAdaptation,minRadius,network,\
		NodeAttribute,nodeList,radius,radiusConstantTime,singlePartition,sizeFactor])
		response=api(url=self.__url+"/isom", PARAMS=PARAMS, method="POST", verbose=verbose)
		return response

	def kamada_kawai(self,defaultEdgeWeight=None,EdgeAttribute=None,\
		m_anticollisionSpringStrength=None,m_averageIterationsPerNode=None,\
		m_disconnectedNodeDistanceSpringRestLength=None,\
		m_disconnectedNodeDistanceSpringStrength=None,m_layoutPass=None,\
		m_nodeDistanceRestLengthConstant=None,m_nodeDistanceStrengthConstant=None,\
		maxWeightCutoff=None,minWeightCutoff=None,network=None,NodeAttribute=None,\
		nodeList=None,randomize=None,singlePartition=None,Type=None,unweighted=None,\
		verbose=None):
		"""
		Execute the Edge-weighted Spring Embedded Layout on a network.

		:param defaultEdgeWeight (string, optional): The default edge weight to con
			sider, default is 0.5
		:param EdgeAttribute (string, optional): The name of the edge column contai
			ning numeric values that will be used as weights in the layout algor
			ithm. Only columns containing numeric values are shown
		:param m_anticollisionSpringStrength (string, optional): Strength to apply
			to avoid collisions, in numeric value
		:param m_averageIterationsPerNode (string, optional): Average number of ite
			ratations for each node, in numeric value
		:param m_disconnectedNodeDistanceSpringRestLength (string, optional): Rest
			length of a 'disconnected' spring, in numeric value
		:param m_disconnectedNodeDistanceSpringStrength (string, optional): Strengt
			h of a 'disconnected' spring, in numeric value
		:param m_layoutPass (string, optional): Number of layout passes, in numeric
				value
		:param m_nodeDistanceRestLengthConstant (string, optional): Spring rest len
			gth, in numeric value
		:param m_nodeDistanceStrengthConstant (string, optional): Spring strength,
			in numeric value
		:param maxWeightCutoff (string, optional): The maximum edge weight to consi
			der, default to the Double.MAX value
		:param minWeightCutoff (string, optional): The minimum edge weight to consi
			der, numeric values, default is 0
		:param network (string, optional): Specifies a network by name, or by SUID
			if the prefix SUID: is used. The keyword CURRENT, or a blank value c
			an also be used to specify the current network.
		:param NodeAttribute (string, optional): The name of the node column contai
			ning numeric values that will be used as weights in the layout algor
			ithm. Only columns containing numeric values are shown
		:param nodeList (string, optional): Specifies a list of nodes. The keywords
				all, selected, or unselected can be used to specify nodes by their
			selection state. The pattern COLUMN:VALUE sets this parameter to any
				rows that contain the specified column value; if the COLUMN prefix
			is not used, the NAME column is matched by default. A list of COLUMN
			:VALUE pairs of the format COLUMN1:VALUE1,COLUMN2:VALUE2,... can be
			used to match multiple values.
		:param randomize (string, optional): Randomize graph before layout; boolean
				values only, true or false; defaults to true
		:param singlePartition (string, optional): Don't partition graph before lay
			out; boolean values only, true or false; defaults to false
		:param Type (string, optional): How to interpret weight values; must be one
				of Heuristic, -Log(value), 1 - normalized value and normalized valu
			e. Defaults to Heuristic = ['Heuristic', '-Log(value)', '1 - normali
			zed value', 'normalized value']
		:param unweighted (string, optional): Use unweighted edges; boolean values
			only, true or false; defaults to false
		"""
		network=check_network(self,network,verbose=verbose)
		PARAMS=set_param(['defaultEdgeWeight','EdgeAttribute',\
		'm_anticollisionSpringStrength','m_averageIterationsPerNode',\
		'm_disconnectedNodeDistanceSpringRestLength',\
		'm_disconnectedNodeDistanceSpringStrength','m_layoutPass',\
		'm_nodeDistanceRestLengthConstant','m_nodeDistanceStrengthConstant',\
		'maxWeightCutoff','minWeightCutoff','network','NodeAttribute','nodeList',\
		'randomize','singlePartition','Type','unweighted'],[defaultEdgeWeight,\
		EdgeAttribute,m_anticollisionSpringStrength,m_averageIterationsPerNode,\
		m_disconnectedNodeDistanceSpringRestLength,\
		m_disconnectedNodeDistanceSpringStrength,m_layoutPass,\
		m_nodeDistanceRestLengthConstant,m_nodeDistanceStrengthConstant,\
		maxWeightCutoff,minWeightCutoff,network,NodeAttribute,nodeList,randomize,\
		singlePartition,Type,unweighted])
		response=api(url=self.__url+"/kamada-kawai", PARAMS=PARAMS, method="POST", verbose=verbose)
		return response


	def set_preferred(self,preferredLayout=None,verbose=None):
		"""
		Sets the preferred layout. Takes a specific name as defined in the API
		Default is grid.

		:param preferredLayout (string, optional): Layout to use as preferred, for
			allowed names see Layout API
		"""
		PARAMS=set_param(['preferredLayout'],[preferredLayout])
		response=api(url=self.__url+"/set preferred", PARAMS=PARAMS, method="POST", verbose=verbose)
		return response


	def stacked_node_layout(self,EdgeAttribute=None,network=None,NodeAttribute=None,\
		nodeList=None,x_position=None,y_start_position=None,verbose=None):
		"""
		Execute the Stacked Node Layout on a network.

		:param EdgeAttribute (string, optional): The name of the edge column contai
			ning numeric values that will be used as weights in the layout algor
			ithm. Only columns containing numeric values are shown
		:param network (string, optional): Specifies a network by name, or by SUID
			if the prefix SUID: is used. The keyword CURRENT, or a blank value c
			an also be used to specify the current network.
		:param NodeAttribute (string, optional): The name of the node column contai
			ning numeric values that will be used as weights in the layout algor
			ithm. Only columns containing numeric values are shown
		:param nodeList (string, optional): Specifies a list of nodes. The keywords
				all, selected, or unselected can be used to specify nodes by their
			selection state. The pattern COLUMN:VALUE sets this parameter to any
				rows that contain the specified column value; if the COLUMN prefix
			is not used, the NAME column is matched by default. A list of COLUMN
			:VALUE pairs of the format COLUMN1:VALUE1,COLUMN2:VALUE2,... can be
			used to match multiple values.
		:param x_position (string, optional): X start position, in numeric value
		:param y_start_position (string, optional): Y start position, in numeric va
			lue
		"""
		network=check_network(self,network,verbose=verbose)
		PARAMS=set_param(['EdgeAttribute','network','NodeAttribute','nodeList',\
		'x_position','y_start_position'],[EdgeAttribute,network,NodeAttribute,\
		nodeList,x_position,y_start_position])
		response=api(url=self.__url+"/stacked-node-layout", PARAMS=PARAMS, method="POST", verbose=verbose)
		return response



from .base import *

class apps(object):
    """
    cytoscape session interface as shown in CyREST's swagger documentation.

    :param url: an url of the type 'http://' + host + ':' + str(port) + '/' + version + '/'.
    """

    def __init__(self, url):
        self.__url = url + 'commands/apps'
        self.___url=url

    def getAppList(self, verbose=None):
        """
        Returns installed Cytoscape Apps that have CyREST accessible Functions or Commands, as a list of App names.

        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'apps', method="H", verbose=verbose, parse_params=False)
        return response


import requests


class SessionClient(object):

    def __init__(self, url):
        self.__url = url + 'session'

    def delete(self):
        requests.delete(self.__url)

    def save(self, file_name=None):
        if file_name is None:
            raise ValueError('Session file name is required.')

        post_url = self.__url
        params = {'file': file_name}
        res = requests.post(post_url, params=params)
        return res

    def open(self, file_name=None):
        if file_name is None:
            raise ValueError('Session file name is required.')

        get_url = self.__url
        params = {'file': file_name}
        res = requests.get(get_url, params=params)
        return res



from .base import *
from .commands import *
from .command import *
from .cybrowser import *
from .session import *
from .network import *
from .node import *
from .vizmap import *
from .diffusion import *
from .idmapper import *
from .edge import *
from .group import *
from .view import *
from .layout import *
from .table import *
from .cyndex2 import *
from .apply import *
from .apps import *
from .styles import *
from .ui import *
from .enrichmentmap import *
from .collections import *
from .networks import *

import json
import copy

class cyclient(object):
    """
    A CyREST client.

    :param host: CyREST host server address. default='localhost'
    :param port: CyREST port number. default=1234
    :param version: CyREST version. default='v1'

    returns: a cyclient object.
    """

    def __init__(self, host=HOST, port=PORT, version=VERSION):
        self.__url = 'http://' + host + ':' + str(port) + '/' + version + '/'
        #self.host=host
        #self.port=port
        #self.version=version
        self.commands=commands(self.__url)
        self.command=command(self.__url)
        self.cybrowser=cybrowser(self.__url)
        self.session=session(self.__url)
        self.network=network(self.__url)
        self.node=node(self.__url)
        self.vizmap=vizmap(self.__url)
        self.diffusion=diffusion(self.__url)
        self.edge=edge(self.__url)
        self.group=group(self.__url)
        self.view=view(self.__url)
        self.layout=layout(self.__url)
        self.table=table(self.__url)
        self.cyndex2=cyndex2(self.__url)
        self.apply=apply(self.__url)
        self.styles=styles(self.__url)
        self.ui=ui(self.__url)
        self.enrichmentmap=enrichmentmap(self.__url)
        self.collections=collections(self.__url)
        self.apps=apps(self.__url)
        self.networks=networks(self.__url)


    def status(self, verbose=False):
        """
        Checks the status of your CyREST server.
        """
        try:
            response=api(url=self.__url, method="GET", verbose=verbose)
        except Exception as e:
            print('Could not get status from CyREST:\n\n' + str(e))
        else:
            print('CyREST online!')

    def copy(self):
        """
        Creates a copy of the cyclient object.
        """
        return copy.deepcopy(self)

    def version(self, verbose=False):
        """
        Checks Cytoscape version
        """
        response=api(url=self.__url+"version",method="H", verbose=verbose)
        response=json.loads(response)
        for k in response.keys():
            print(k, response[k])

    def result(self, filetype="PNG", saveas=None, verbose=False):
        """
        Checks the current network. 
            
        :param filetype: file type eg.PDF, PNG, CYS, CYJS; default="PNG" 
        :param saveas: /path/to/non/tmp/file.prefix
        :param host: cytoscape host address, default=cytoscape_host
        :param port: cytoscape port, default=1234
        :returns: an image
        """
        from wand.image import Image as WImage
        from shutil import copyfile
        import tempfile
        from time import sleep
        import os

        sleep(1)
        u=self.__url 
        host=u.split("//")[1].split(":")[0]
        port=u.split(":")[2].split("/")[0]
        version=u.split(":")[2].split("/")[1]

        def MAKETMP():
            (fd, tmp_file) = tempfile.mkstemp()
            tmp_dir=tempfile.gettempdir()
            tmp_file=tmp_dir+"/"+tmp_file.split("/")[-1]
            return tmp_file
        
        outfile=MAKETMP()
        
        extensions={"PNG":".png","PDF":".pdf","CYS":".cys","CYJS":".cyjs"}
        ext=extensions[filetype]
        
        response=api("view","fit content",host=host,port=port, version=version, verbose=verbose)
        sleep(2)
        response=api("view", "export" , {"options":filetype,"outputFile":outfile, "view":"current"}, host=host,port=port,version=version,verbose=verbose)
        if host!='localhost':
            import paramiko
            print("Looking to ssh keys for remote access.")
            ssh = paramiko.SSHClient()
            ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
            ssh.connect(host)
            ftp_client=ssh.open_sftp()
            ftp_client.get(outfile+ext,outfile+ext)
            ssh_stdin, ssh_stdout, ssh_stderr = ssh.exec_command("rm "+outfile+ext )
            
        img = WImage(filename=outfile+ext)
        if saveas:
            copyfile(outfile+ext,saveas)
        os.remove(outfile+ext)
        return img
        



# -*- coding: utf-8 -*-

def check_response(res):
    """ Check HTTP response and raise exception if response is not OK. """
    try:
        res.raise_for_status() # Alternative is res.ok
    except Exception as exc:
        # Bad response code, e.g. if adding an edge with nodes that doesn't exist
        try:
            err_info = res.json()
            err_msg = err_info['message'] # or 'localizeMessage'
        except ValueError:
            err_msg = res.text[:40] # Take the first 40 chars of the response
        except KeyError:
            err_msg = res.text[:40] + ("(No 'message' in err_info dict: %s"
                                       % list(err_info.keys()))
        exc.args += (err_msg,)
        raise exc






from .base import *

class collections(object):
    """
    cytoscape session interface as shown in CyREST's swagger documentation.

    :param url: an url of the type 'http://' + host + ':' + str(port) + '/' + version + '/'.
    """

    def __init__(self, url):
        self.__url = url + 'commands/collections'
        self.___url=url


    def getCollectionCount(self, verbose=None):
        """
        Returns a count of all root networks.

        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'collections/count', method="H", verbose=verbose, parse_params=False)
        return response


from .base import *

class network(object):
    """
    cytoscape network interface as shown in CyREST's swagger documentation for 'network'.

    :param url: an url of the type 'http://' + host + ':' + str(port) + '/' + version + '/'.
    """

    def __init__(self, url):
        self.__url = url + 'commands/network'

    def add(self, edgeList=None, network=None, nodeList=None, verbose=False):
        """
        Adds nodes and edges to an existing network. The nodes and edges to be
        added must already exist in the network collection. This command is
        most often used to populate a subnetwork with selected nodes and edges
        from a parent network.

        :param edgeList: (string, optional), Specifies a list of edges. The
            keywords all, selected, or unselected can be used to specify edges
            by their selection state. The pattern COLUMN:VALUE sets this parameter
            to any rows that contain the specified column value; if the COLUMN
            prefix is not used, the NAME column is matched by default. A list
            of COLUMN:VALUE pairs of the format COLUMN1:VALUE1,COLUMN2:VALUE2,...
            can be used to match multiple values.
        :param network: (string, optional), Specifies a network by name, or by
            SUID if the prefix SUID: is used. The keyword CURRENT, or a blank value
            can also be used to specify the current network.
        :param nodeList: (string, optional), Specifies a list of nodes. The keywords all,
            selected, or unselected can be used to specify nodes by their selection
            state. The pattern COLUMN:VALUE sets this parameter to any rows that
            contain the specified column value; if the COLUMN prefix is not used,
            the NAME column is matched by default. A list of COLUMN:VALUE pairs of
            the format COLUMN1:VALUE1,COLUMN2:VALUE2,... can be used to match multiple
            values.
        :param verbose: print more
        """
        network=check_network(self,network,verbose=verbose)
        PARAMS=set_param(["edgeList","network","nodeList"],[edgeList,network,nodeList])
        response=api(url=self.__url+"/add", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response


    def add_edge(self, isDirected=None,name=None,network=None,sourceName=None,targetName=None, verbose=False):
        """
        Add a new edge between two existing nodes in a network. The names of the
        nodes must be specified and much match the value in the 'name' column for
        each node.

        :param isDirected (string, optional): Whether the edge should be directed
            or not. Even though all edges in Cytoscape have a source and target, by
            default they are treated as undirected. Setting this to 'true' will
            flag some algorithms to treat them as directed, although many current
            implementations will ignore this flag.
        :param name (string, optional): Set the 'name' and 'shared name' columns
            for this edge to the provided value. ,
        :param network (string, optional): Specifies a network by name, or by SUID
            if the prefix SUID: is used. The keyword CURRENT, or a blank value can
            also be used to specify the current network.
        :param sourceName (string): Enter the name of an existing node in the
            network to be the source of the edge. Note that this is the name as
            defined in the 'name' column of the network.
        :param targetName (string): Enter the name of an existing node in the
            network to be the target of the edge. Note that this is the name as
            defined in the 'name' column of the network.
        :param verbose: print more
        """
        network=check_network(self,network,verbose=verbose)
        PARAMS=set_param(["isDirected","name","network","sourceName","targetName"],\
        [isDirected,name,network,sourceName,targetName])
        response=api(url=self.__url+"/add edge", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response

    

    def add_node(self, name=None,network=None, verbose=False):
        """
        Add a new node to an existing network. The name of the node must be provided.

        :param name (string, optional): The name of the node, which will be assigned
            to both the 'name' and 'shared name' columns
        :param network (string, optional): Specifies a network by name, or by SUID
            if the prefix SUID: is used. The keyword CURRENT, or a blank value can
            also be used to specify the current network.
        :param verbose: print more

        :returns: node SUID
        """
        network=check_network(self,network,verbose=verbose)
        PARAMS=set_param(["name","network"],[name,network])
        response=api(url=self.__url+"/add node", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response

    def clone(self, network=None, verbose=False):
        """
        Create a new network by cloning an existing network. The new network will
        be created as part of a new network collection. The SUID of the new network
        and view (if one is created) are returned.

        :param name (string, optional): The name of the node, which will be assigned
            to both the 'name' and 'shared name' columns
        :param network (string, optional): Specifies a network by name, or by SUID
            if the prefix SUID: is used. The keyword CURRENT, or a blank value can
            also be used to specify the current network.
        :param verbose: print more

        :returns: { network, view }
        """
        network=check_network(self,network,verbose=verbose)
        PARAMS=set_param(["network"], [network])
        response=api(url=self.__url+"/clone", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response

    def connect_nodes(self, network=None, nodes=None, verbose=False):
        """
        Create new edges that connect a list of nodes.

        :param network (string): Specifies a network by name, or by SUID if the
            prefix SUID: is used. The keyword CURRENT, or a blank value can also be
            used to specify the current network.
        :param nodes (string): Specifies a list of nodes. The keywords all,
            selected, or unselected can be used to specify nodes by their selection
            state. The pattern COLUMN:VALUE sets this parameter to any rows that
            contain the specified column value; if the COLUMN prefix is not used,
            the NAME column is matched by default. A list of COLUMN:VALUE pairs of
            the format COLUMN1:VALUE1,COLUMN2:VALUE2,... can be used to match multiple values.
        :param verbose: print more

        :returns: [ list of generated edges ]
        """
        network=check_network(self,network,verbose=verbose)
        PARAMS=set_param(["nodes","network"],[nodes,network])
        response=api(url=self.__url+"/connect nodes", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response

    def create(self, edgeList=None, excludeEdges=None, networkName=None, nodeList=None, source=None, verbose=False):
        """
        Create a new network from a list of nodes and edges in an existing source network.
        The SUID of the network and view are returned.

        :param edgeList (string, optional): Specifies a list of edges. The keywords
            all, selected, or unselected can be used to specify edges by their
            selection state. The pattern COLUMN:VALUE sets this parameter to any
            rows that contain the specified column value; if the COLUMN prefix is
            not used, the NAME column is matched by default. A list of COLUMN:VALUE
            pairs of the format COLUMN1:VALUE1,COLUMN2:VALUE2,... can be used to
            match multiple values.
        :param excludeEdges (string, optional): Unless this is set to true, edges
            that connect nodes in the nodeList are implicitly included
        :param networkName (string, optional):
        :param nodeList (string, optional): Specifies a list of nodes. The keywords
            all, selected, or unselected can be used to specify nodes by their
            selection state. The pattern COLUMN:VALUE sets this parameter to any
            rows that contain the specified column value; if the COLUMN prefix is
            not used, the NAME column is matched by default. A list of COLUMN:VALUE
            pairs of the format COLUMN1:VALUE1,COLUMN2:VALUE2,... can be used to
            match multiple values.
        :param source (string, optional): Specifies a network by name, or by SUID
            if the prefix SUID: is used. The keyword CURRENT, or a blank value can
            also be used to specify the current network.
        :param verbose: print more

        :returns: { netowrk, view }
        """
        network=check_network(self,source, verbose=verbose)
        PARAMS=set_param(["edgeList","excludeEdges","networkName","nodeList","source"], \
        [edgeList,excludeEdges,networkName,nodeList,network])
        response=api(url=self.__url+"/create", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response

    def create_attribute(self, column=None, listType=None, namespace=None, network=None, atype=None, verbose=False):
        """
        Creates a new network column.

        :param column (string, optional): Unique name of column
        :param listType (string, optional): Can be one of integer, long, double,
            or string. = ['integer', 'long', 'double', 'string', 'boolean']
        :param namespace (string, optional): Node, Edge, and Network objects
            support the default, local, and hidden namespaces. Root networks also
            support the shared namespace. Custom namespaces may be specified
            by Apps.
        :param network (string, optional): Specifies a network by name, or by
            SUID if the prefix SUID: is used. The keyword CURRENT, or a blank value
            can also be used to specify the current network.
        :param atype (string, optional): Can be one of integer, long, double,
            string, or list. = ['integer', 'long', 'double', 'string', 'boolean',
            'list']
        :param verbose: print more

        :returns: { columnName }
        """
        network=check_network(self,network,verbose=verbose)
        PARAMS=set_param(["column","listType","namespace","network","type"], \
        [column,listType,namespace,network,atype])
        response=api(url=self.__url+"/create attribute", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response

    

    def create_empty(self, name=None, renderers=None, RootNetworkList=None, verbose=False):
        """
        Create a new, empty network. The new network may be created as part of
        an existing network collection or a new network collection.

        :param name (string, optional): Enter the name of the new network.
        :param renderers (string, optional): Select the renderer to use for the
            new network view. By default, the standard Cytoscape 2D renderer (Ding)
            will be used = [''],
        :param RootNetworkList (string, optional): Choose the network collection
            the new network should be part of. If no network collection is selected,
            a new network collection is created. = [' -- Create new network collection --',
            'cy:command_documentation_generation']
        :param verbose: print more
        """
        PARAMS=set_param(["name","renderers","RootNetworkList"],[name,renderers,RootNetworkList])
        response=api(url=self.__url+"/create empty", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response

    

    def delete(self, edgeList=None, network=None, nodeList=None, verbose=False):
        """
        Deletes nodes and edges provided by the arguments, or if no nodes or edges are provides,
        the selected nodes and edges. When deleting nodes, adjacent edges are also deleted.

        :param edgeList (string, optional): Specifies a list of edges. The keywords
            all, selected, or unselected can be used to specify edges by their
            selection state. The pattern COLUMN:VALUE sets this parameter to any
            rows that contain the specified column value; if the COLUMN prefix is
            not used, the NAME column is matched by default. A list of COLUMN:VALUE
            pairs of the format COLUMN1:VALUE1,COLUMN2:VALUE2,... can be used to match
            multiple values.
        :param network (string, optional): Specifies a network by name, or by SUID
            if the prefix SUID: is used. The keyword CURRENT, or a blank value can
            also be used to specify the current network.
        :param nodeList (string, optional): Specifies a list of nodes. The keywords all,
            selected, or unselected can be used to specify nodes by their selection state.
            The pattern COLUMN:VALUE sets this parameter to any rows that contain the specified
            column value; if the COLUMN prefix is not used, the NAME column is matched by default.
            A list of COLUMN:VALUE pairs of the format COLUMN1:VALUE1,COLUMN2:VALUE2,... can be
            used to match multiple values.
        :param verbose: print more
        """
        network=check_network(self,network,verbose=verbose)
        PARAMS=set_param(["edgeList","network","nodeList"],[edgeList,network,nodeList])
        response=api(url=self.__url+"/delete", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response


    def deselect(self, edgeList=None, network=None, nodeList=None, verbose=False):
        """
        Deselect nodes and/or edges in a network. A list of nodes and/or edges
        may be provided and those nodes and edges will be deselected.

        :param edgeList (string, optional): Specifies a list of edges. The keywords all,
            selected, or unselected can be used to specify edges by their selection state.
            The pattern COLUMN:VALUE sets this parameter to any rows that contain the
            specified column value; if the COLUMN prefix is not used, the NAME column
            is matched by default. A list of COLUMN:VALUE pairs of the format
            COLUMN1:VALUE1,COLUMN2:VALUE2,... can be used to match multiple values.
        :param network (string, optional): Specifies a network by name, or by SUID
            if the prefix SUID: is used. The keyword CURRENT, or a blank value can
            also be used to specify the current network.
        :param nodeList (string, optional): Specifies a list of nodes. The keywords
            all, selected, or unselected can be used to specify nodes by their
            selection state. The pattern COLUMN:VALUE sets this parameter to any
            rows that contain the specified column value; if the COLUMN prefix is
            not used, the NAME column is matched by default. A list of COLUMN:VALUE
            pairs of the format COLUMN1:VALUE1,COLUMN2:VALUE2,... can be used to
            match multiple values.
        :param verbose: print more

        :retunrs: [ list of deselected nodes and edges ]
        """
        network=check_network(self,network,verbose=verbose)
        PARAMS=set_param(["edgeList","network","nodeList"],[edgeList,network,nodeList])
        response=api(url=self.__url+"/deselect", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response

    def destroy(self, network=None, verbose=False):
        """
        Destroy (delete) a network. The SUID of the destroyed network is returned.

        :param network (string, optional): Specifies a network by name, or by SUID
            if the prefix SUID: is used. The keyword CURRENT, or a blank value can
            also be used to specify the current network.
        :param verbose: print more

        :returns: id of destroyed network
        """
        network=check_network(self,network,verbose=verbose)
        PARAMS=set_param(["network"],[network])
        response=api(url=self.__url+"/destroy", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response

    def export(self, options=None, OutputFile=None, verbose=False):
        """
        Export a network to a network file (e.g. XGMML, SIF, etc.)

        :param options (string, optional): The format of the output file. = ['CX JSON (*.cx)',
            'Cytoscape.js JSON (*.cyjs)', 'GraphML files (*.graphml, *.xml)', 'NNF (*.nnf)',
            'PSI-MI Level 1', 'PSI-MI Level 2.5', 'SIF (*.sif)', 'XGMML (*.xgmml, *.xml)']
        :param OutputFile (string, optional):
        :param verbose: print more

        :returns: { file }
        """
        PARAMS=set_param(["options","OutputFile"],[options,OutputFile])
        response=api(url=self.__url+"/export", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response

    def get(self, network=None, verbose=False):
        """
        Specifies a network by name, or by SUID if the prefix SUID: is used.
        The keyword CURRENT, or a blank value can also be used to specify the
        current network.

        This function sets the self.network.network_name and
        self.network.network_suid of a cyclient object.

        :param network: network (string, optional): Specifies a network by name,
            or by SUID if the prefix SUID: is used. The keyword CURRENT, or a blank
            value can also be used to specify the current network.
        :param verbose: print more

        :returns: { dictionary with values for the respective network }
        """

        network=check_network(self,network,verbose=verbose)
        PARAMS=set_param(["network"],[network])
        response=api(url=self.__url+"/get", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response

    def get_attribute(self, columnList=None, namespace=None, network=None, verbose=False):
        """
        Returns the attributes for the network passed as parameter.

        :param columnList (string, optional): A list of column names, separated by commas.
        :param namespace (string, optional): Node, Edge, and Network objects support
            the default, local, and hidden namespaces. Root networks also support the
            shared namespace. Custom namespaces may be specified by Apps.
        :param network (string, optional): Specifies a network by name, or by
            SUID if the prefix SUID: is used. The keyword CURRENT, or a blank
            value can also be used to specify the current network.
        :param verbose: print more

        :returns: { attributes for the network passed in column list }
        """
        network=check_network(self,network,verbose=verbose)
        PARAMS=set_param(["columnList","namespace","network"],[columnList,namespace,network])
        response=api(url=self.__url+"/get attribute", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response

    def get_porperties(self, network=None, propertyList=None, verbose=False):
        """
        Returns the visual properties for the network that matches the passed parameters.

        :param network (string): Specifies a network by name, or by SUID if the
            prefix SUID: is used. The keyword CURRENT, or a blank value can also
            be used to specify the current network.
        :param propertyList (string): A comma-separated list of network properties
        :param verbose: print more

        :returns: visual properties for the network that matches the passed parameters
        """
        network=check_network(self,network,verbose=verbose)
        PARAMS=set_param(["network","propertyList"],[network,propertyList])
        response=api(url=self.__url+"/get properties", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response

    def hide(self, edgeList=None, network=None, nodeList=None, verbose=False):
        """
        Hide nodes and/or edges in a network. A list of nodes and/or edges may
        be provided and those nodes and edges will be hidden in the view associated
        with the provided network.Note that the network '''must''' have a view.
        The SUIDs of the hidden nodes and/or edges are returned.

        :param edgeList (string, optional): Specifies a list of edges. The keywords
            all, selected, or unselected can be used to specify edges by their
            selection state. The pattern COLUMN:VALUE sets this parameter to any
            rows that contain the specified column value; if the COLUMN prefix is
            not used, the NAME column is matched by default. A list of COLUMN:VALUE
            pairs of the format COLUMN1:VALUE1,COLUMN2:VALUE2,... can be used to
            match multiple values.
        :param network (string, optional): Specifies a network by name, or by SUID
            if the prefix SUID: is used. The keyword CURRENT, or a blank value can
            also be used to specify the current network.
        :param nodeList (string, optional): Specifies a list of nodes. The keywords
            all, selected, or unselected can be used to specify nodes by their
            selection state. The pattern COLUMN:VALUE sets this parameter to any
            rows that contain the specified column value; if the COLUMN prefix is
            not used, the NAME column is matched by default. A list of COLUMN:VALUE
            pairs of the format COLUMN1:VALUE1,COLUMN2:VALUE2,... can be used to
            match multiple values.
        :param verbose: print more

        :returns: [ list of SUIDs of the hidden nodes and/or edges ]
        """
        network=check_network(self,network,verbose=verbose)
        PARAMS=set_param(["network","edgeList","nodeList"],[network,edgeList,nodeList])
        response=api(url=self.__url+"/hide", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response


    def import_file(self, dataTypeList=None, defaultInteraction=None, delimiters=None, \
        delimitersForDataList=None, afile=None, firstRowAsColumnNames=None, \
        indexColumnSourceInteraction=None, indexColumnTargetInteraction=None, indexColumnTypeInteraction=None, \
        NetworkViewRendererList=None, RootNetworkList=None, startLoadRow=None,\
        TargetColumnList=None, verbose=False):
        """
        Import a new network from a tabular formatted file type (e.g. csv, tsv,
        Excel, etc.). Use network load file to load network formatted files. This
        command will create a new network collection if no current network collection
        is selected, otherwise it will add the network to the current collection. The
        SUIDs of the new networks and views are returned.

        :param dataTypeList (string, optional): List of column data types ordered
            by column index (e.g. "string,int,long,double,boolean,intlist" or
            just "s,i,l,d,b,il"): ,
        :param defaultInteraction (string, optional): Used to set the default
            interaction type to use when there is no interaction type column.
        :param delimiters (string, optional): Select the delimiters to use to
            separate columns in the table, from the list ',',' ','TAB', or ';'.
            TAB and ',' are used by default = [',', ';', ' ', '\t']
        :param delimitersForDataList (string, optional): Select the delimiters
            to use to separate list entries in a list, from the list '|','\','/',
            or ','. | is used by default = ['\|', '\', '/', ',']
        :param afile (string): The path to the file that contains the table or
            network to be imported.
        :param firstRowAsColumnNames (string, optional): If this is true then
            the first row should contain the names of the columns. Note that
            startLoadRow must be set for this to work properly
        :param indexColumnSourceInteraction (string): The column index that
            contains the source node identifiers.
        :param indexColumnTargetInteraction (string, optional): The column index
            that contains the target node identifiers. If this is not specified
            then the resulting network will have no edges
        :param indexColumnTypeInteraction (string, optional): The column index
            that contains the interaction type. This is not required.
        :param NetworkViewRendererList (string, optional): Enter the network
            view renderer that this network should use. This is only useful if
            multiple renderers have been installed, which is rare. = ['']
        :param RootNetworkList (string, optional): The name of the network
            collection (root network) that the imported network should be part of.
            A name of -- Create new network collection -- will result in the
            creation of a new network collection for this import. = ['-- Create
            new network collection --', 'cy:command_documentation_generation']
        :param startLoadRow (string, optional): The starting row of the import.
            This is used to skip over comments and other non-data rows at the
            beginning of the file.
        :param TargetColumnList (string, optional): Enter the name of the column
            in the existing network collection (root network) that you want to
            map your input identifiers to. = ['']
        :param verbose: print more

        :returns:  { SUIDs of the new networks and views }
        """

        PARAMS=set_param(["dataTypeList","defaultInteraction","delimiters","delimitersForDataList",\
        "file","firstRowAsColumnNames","indexColumnSourceInteraction","indexColumnTargetInteraction",\
        "indexColumnTypeInteraction","NetworkViewRendererList","RootNetworkList","startLoadRow","TargetColumnList"],\
        [dataTypeList,defaultInteraction,delimiters,delimitersForDataList,\
        afile,firstRowAsColumnNames,indexColumnSourceInteraction,indexColumnTargetInteraction,\
        indexColumnTypeInteraction,NetworkViewRendererList,RootNetworkList,startLoadRow,TargetColumnList])
        response=api(url=self.__url+"/import file", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response

    def import_url(self, dataTypeList=None, defaultInteraction=None, delimiters=None, \
        delimitersForDataList=None, firstRowAsColumnNames=None, \
        indexColumnSourceInteraction=None, indexColumnTargetInteraction=None, indexColumnTypeInteraction=None, \
        NetworkViewRendererList=None, RootNetworkList=None, startLoadRow=None,\
        TargetColumnList=None, url=None, verbose=False):
        """
        Import a new network from a tabular formatted file type (e.g. csv, tsv,
        Excel, etc.). Use network load file to load network formatted files. This
        command will create a new network collection if no current network collection
        is selected, otherwise it will add the network to the current collection. The
        SUIDs of the new networks and views are returned.

        :param dataTypeList (string, optional): List of column data types ordered
            by column index (e.g. "string,int,long,double,boolean,intlist" or just
            "s,i,l,d,b,il"): ,
        :param defaultInteraction (string, optional): Used to set the default
            interaction type to use when there is no interaction type column.
        :param delimiters (string, optional): Select the delimiters to use to
            separate columns in the table, from the list ',',' ','TAB', or ';'.
            TAB and ',' are used by default = [',', ';', ' ', '\t']
        :param delimitersForDataList (string, optional): Select the delimiters
            to use to separate list entries in a list, from the list '|','\','/',
            or ','. | is used by default = ['\|', '\', '/', ',']
        :param firstRowAsColumnNames (string, optional): If this is true then
            the first row should contain the names of the columns. Note that
            startLoadRow must be set for this to work properly
        :param indexColumnSourceInteraction (string): The column index that
            contains the source node identifiers.
        :param indexColumnTargetInteraction (string, optional): The column index
            that contains the target node identifiers. If this is not specified
            then the resulting network will have no edges
        :param indexColumnTypeInteraction (string, optional): The column index
            that contains the interaction type. This is not required.
        :param NetworkViewRendererList (string, optional): Enter the network view
            renderer that this network should use. This is only useful if multiple
            renderers have been installed, which is rare. = ['']
        :param RootNetworkList (string, optional): The name of the network
            collection (root network) that the imported network should be part of.
            A name of -- Create new network collection -- will result in the creation
            of a new network collection for this import. = ['-- Create new network collection --',
            'cy:command_documentation_generation']
        :param startLoadRow (string, optional): The starting row of the import.
            This is used to skip over comments and other non-data rows at the
            beginning of the file.
        :param TargetColumnList (string, optional): Enter the name of the
            column in the existing network collection (root network) that you
            want to map your input identifiers to. = ['']
        :param url (string): The URL of the file or resource that provides the
            table or network to be imported.
        :param verbose: print more

        :returns:  { SUIDs of the new networks and views }
        """

        PARAMS=set_param(["dataTypeList","defaultInteraction","delimiters","delimitersForDataList",\
        "firstRowAsColumnNames","indexColumnSourceInteraction","indexColumnTargetInteraction",\
        "indexColumnTypeInteraction","NetworkViewRendererList","RootNetworkList","startLoadRow","TargetColumnList","url"],\
        [dataTypeList,defaultInteraction,delimiters,delimitersForDataList,\
        firstRowAsColumnNames,indexColumnSourceInteraction,indexColumnTargetInteraction,\
        indexColumnTypeInteraction,NetworkViewRendererList,RootNetworkList,startLoadRow,TargetColumnList, url])
        response=api(url=self.__url+"/import url", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response

    def list(self, verbose=False):
        """
        List all of the networks in the current session.

        :param verbose: print more

        :returns: [ list of network suids ]
        """

        response=api(url=self.__url+"/list", method="POST", verbose=verbose)
        return response

    def list_attributes(self, namespace=None, network=None, verbose=False):
        """
        Returns a list of column names assocated with a network.

        :param namespace (string, optional): Node, Edge, and Network objects
            support the default, local, and hidden namespaces. Root networks also
                support the shared namespace. Custom namespaces may be specified
                by Apps.
        :param network (string, optional): Specifies a network by name, or by
            SUID if the prefix SUID: is used. The keyword CURRENT, or a blank
            value can also be used to specify the current network.
        :param verbose: print more

        :returns: [ list of column names assocated with a network ]
        """
        network=check_network(self,network,verbose=verbose)
        PARAMS=set_param(["namespace","network"],[namespace,network])
        response=api(url=self.__url+"/list attributes", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response


    def list_properties(self, network=None, verbose=False):
        """
        List all of the visual properties for networks.

        :param network (string, optional): Specifies a network by name, or by
            SUID if the prefix SUID: is used. The keyword CURRENT, or a blank
            value can also be used to specify the current network.
        :param verbose: print more

        :returns: [ List all of the visual properties ]
        """
        network=check_network(self,network,verbose=verbose)
        PARAMS=set_param(["network"],[network])
        response=api(url=self.__url+"/list properties", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response


    def load_file(self, afile=None, verbose=False):
        """
        Load a new network from a network file type (e.g. SIF, XGMML, etc.).
        Use network import file to load networks from Excel or csv files. This
        command will create a new network collection if no current network collection
        is selected, otherwise it will add the network to the current collection.
        The SUIDs of the new networks and views are returned.

        :param afile (string): Select a network format file. This command does
            not support csv or Excel files. Use network import file for that.
        :param verbose: print more

        :returns: { SUIDs of the new networks and views }
        """
        PARAMS=set_param(["file"],[afile])
        response=api(url=self.__url+"/load file", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response

    def load_url(self, url=None, verbose=False):
        """
        Load a new network from a URL that points to a network file type (e.g.
        SIF, XGMML, etc.). Use network import url to load networks from Excel or
        csv files. This command will create a new network collection if no current
        network collection is selected, otherwise it will add the network to the
        current collection. The SUIDs of the new networks and views are returned.

        :param url (string): Select a URL that points to a network format file.
            This command does not support csv or Excel files. Use network import
            url for that.
        :param verbose: print more

        :returns: { SUIDs of the new networks and views }
        """
        PARAMS=set_param(["url"],[url])
        response=api(url=self.__url+"/load url", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response

    def rename(self, name=None, sourceNetwork=None, verbose=False):
        """
        Rename an existing network. The SUID of the network is returned

        :param name (string): Enter a new title for the network
        :param sourceNetwork (string): Specifies a network by name, or by SUID
            if the prefix SUID: is used. The keyword CURRENT, or a blank value
            can also be used to specify the current network.
        :param verbose: print more

        :returns: SUID of the network is returned
        """
        sourceNetwork=check_network(self,sourceNetwork,verbose=verbose)
        PARAMS=set_param(["name","sourceNetwork"],[name,sourceNetwork])
        response=api(url=self.__url+"/rename", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response

    def select(self, adjacentEdges=None, edgeList=None, extendEdges=None, firstNeighbors=None, \
        invert=None, network=None, nodeList=None, verbose=False):
        """
        Select nodes and/or edges in a network. This command provides options to
        invert the selection, add first neighbors, add adjacent edges of selected
        nodes, and add adjacent nodes of selected edges

        :param adjacentEdges (string, optional): If 'true', then select any edges
            adjacent to any selected nodes. This happens before any inversion
        :param edgeList (string, optional): Specifies a list of edges. The keywords
            all, selected, or unselected can be used to specify edges by their
            selection state. The pattern COLUMN:VALUE sets this parameter to any
            rows that contain the specified column value; if the COLUMN prefix is
            not used, the NAME column is matched by default. A list of COLUMN:VALUE
            pairs of the format COLUMN1:VALUE1,COLUMN2:VALUE2,... can be used to
            match multiple values.
        :param extendEdges (string, optional): If 'true', then select any nodes
            adjacent to any selected edges. This happens before any inversion
        :param firstNeighbors (string, optional): If this option is anything other
            than 'none', add nodes to the selection based on the value of the
            argument. If 'incoming', add nodes to the selection that have edges
            pointing to one of the selected nodes. If 'output', add nodes to the
            selection that have edges that point to them from one of the selected
            nodes. If 'undirected' add any neighbors that have undirected edges
            connecting to any of the selected nodes. Finally, if 'any', then add
            all first neighbors to the selection list. = ['none', 'incoming',
            'outgoing', 'undirected', 'any'],
        :param invert (string, optional): If this option is not 'none', then the
            selected nodes or edges (or both) will be deselected and all other
            nodes or edges will be selected = ['none', 'nodes', 'edges', 'both']
        :param network (string, optional): Specifies a network by name, or by
            SUID if the prefix SUID: is used. The keyword CURRENT, or a blank value
            can also be used to specify the current network.
        :param nodeList (string, optional): Specifies a list of nodes. The keywords
            all, selected, or unselected can be used to specify nodes by their
            selection state. The pattern COLUMN:VALUE sets this parameter to any
            rows that contain the specified column value; if the COLUMN prefix is
            not used, the NAME column is matched by default. A list of COLUMN:VALUE
            pairs of the format COLUMN1:VALUE1,COLUMN2:VALUE2,... can be used to
            match multiple values.
        :param verbose: print more

        :returns: [ list of selected edges and nodes ]
        """
        network=check_network(self,network,verbose=verbose)
        PARAMS=set_param(["adjacentEdges","edgeList","extendEdges","firstNeighbors",\
        "invert","network","nodeList"], \
        [adjacentEdges,edgeList,extendEdges,firstNeighbors,\
        invert,network,nodeList])
        response=api(url=self.__url+"/select", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response

    def set_attribute(self, columnList=None, namespace=None, network=None, valueList=None, verbose=False):
        """
        Set a value in the network table.

        :param columnList (string, optional): A list of column names, separated
            by commas.
        :param namespace (string, optional): Node, Edge, and Network objects
            support the default, local, and hidden namespaces. Root networks also
                support the shared namespace. Custom namespaces may be specified by Apps.
        :param network (string, optional): Specifies a network by name, or by
            SUID if the prefix SUID: is used. The keyword CURRENT, or a blank
            value can also be used to specify the current network.
        :param valueList (string, optional): A list of values, separated by commas.
            List values can be included using the format [value1,value2].
        :param verbose: print more
        """
        network=check_network(self,network,verbose=verbose)
        PARAMS=set_param(["columnList","namespace","network","valueList"], \
        [columnList,namespace,network,valueList])
        response=api(url=self.__url+"/set attribute", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response
 

    def set_current(self, network=None, verbose=False):
        """
        Sets the current network, which can also be null.

        This will set the .network.network_name value to network.

        :param network (string, optional): Specifies a network by name, or by
            SUID if the prefix SUID: is used. The keyword CURRENT, or a blank
            value can also be used to specify the current network.
        :param verbose: print more
        """
        PARAMS=set_param(["network"], [network])
        response=api(url=self.__url+"/set current", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response

    def set_properties(self, network=None, propertyList=None, valueList=None, verbose=False):
        """
        Set network visual properties.

        :param network (string, optional): Specifies a network by name, or by
            SUID if the prefix SUID: is used. The keyword CURRENT, or a blank
            value can also be used to specify the current network.
        :param propertyList (string): A comma-separated list of network properties
        :param valueList (string): A comma-separated list of property values.
            This list must have the same number of elements as the propertyList.
            Each value will be applied to the property specified in the same
            position in the propertyList
        :param verbose: print more
        """
        network=check_network(self,network,verbose=verbose)
        PARAMS=set_param(["network","propertyList","valueList"], \
        [network,propertyList,valueList])
        response=api(url=self.__url+"/set properties", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response

    def show(self, edgeList=None, network=None, nodeList=None, verbose=False):
        """
        Show nodes and/or edges in a network. A list of nodes and/or edges may
        be provided and those nodes and edges will be unhidden in the view associated
        with the provided network.Note that the network '''must''' have a view.
        The SUIDs of the unhidden nodes and/or edges are returned.

        :param edgeList (string, optional): Specifies a list of edges. The keywords
            all, selected, or unselected can be used to specify edges by their
            selection state. The pattern COLUMN:VALUE sets this parameter to any
            rows that contain the specified column value; if the COLUMN prefix is
            not used, the NAME column is matched by default. A list of COLUMN:VALUE
            pairs of the format COLUMN1:VALUE1,COLUMN2:VALUE2,... can be used to match
            multiple values.
        :param network (string, optional): Specifies a network by name, or by SUID
            if the prefix SUID: is used. The keyword CURRENT, or a blank value can
            also be used to specify the current network.
        :param nodeList (string, optional): Specifies a list of nodes. The keywords
            all, selected, or unselected can be used to specify nodes by their
            selection state. The pattern COLUMN:VALUE sets this parameter to any
            rows that contain the specified column value; if the COLUMN prefix is
            not used, the NAME column is matched by default. A list of COLUMN:VALUE
            pairs of the format COLUMN1:VALUE1,COLUMN2:VALUE2,... can be used to
            match multiple values.
        :param verbose: print more

        :returns: [ list of SUIDs of the unhidden nodes and/or edges ]
        """
        network=check_network(self,network,verbose=verbose)
        PARAMS=set_param(["edgeList","network","nodeList"],[edgeList,network,nodeList])
        response=api(url=self.__url+"/show", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response



import numpy as np


def from_ndarray(data, name=None, labels=None, directed=False, weighted=False):
    mat_dim = data.shape
    if mat_dim[0] != mat_dim[1]:
        raise ValueError('Data should be square matrix.')
    data_size = mat_dim[0]

    if labels is not None:
        label_len = len(labels)
        if label_len != data_size:
            raise ValueError('Label length is not equal to the size of data.')

    network_name = name
    if network_name is None:
        network_name = 'from ndarray'

    g = {
        'data': {
            'name': network_name
        },
        'elements': {
            'nodes': [],
            'edges': []
        }
    }

    g['elements']['nodes'] = __get_nodes(labels, data_size)

    if weighted:
        g['elements']['edges'] = __get_weighted_edges(matrix=data)
    else:
        g['elements']['edges'] = __get_unweighted_edges(matrix=data)

    return g

def __get_nodes(labels, size):

    nodes = []
    if labels is None:
        node_labels = np.arange(size)
    else:
        node_labels = labels

    for idx, label in enumerate(node_labels):
        nodes.append(__get_node(idx, label))

    return nodes


def __get_node(node_id, name):
    n = {
        'data': {
            'id': str(node_id),
            'name': str(name)
        }
    }
    return n


def __get_egdes(matrix, labels):
    pass


def __get_edge(source, target, weight=None):
    e = {
        'data': {
            'id': source + '-' + target,
            'source': source,
            'target': target
        }
    }

    if weight is not None:
        e['data']['weight'] = weight

    return e


def __get_unweighted_edges(matrix):
    size = matrix.shape[0]

    edges = []
    row_idx = 0
    for row in matrix:
        idx = row_idx
        while idx < size:
            if row[idx] == 1:
                e = __get_edge(str(row_idx), str(idx))
                edges.append(e)
            idx += 1
        row_idx += 1
    return edges


def __get_weighted_edges(matrix):
    size = matrix.shape[0]
    edges = []
    row_idx = 0
    for row in matrix:
        idx = row_idx
        while idx < size:
            if not np.isnan(row[idx]):
                e = __get_edge(str(row_idx), str(idx), weight=row[idx])
                edges.append(e)
            idx += 1
        row_idx += 1
    return edges
    pass





from .base import *

class session(object):
    """
    cytoscape session interface as shown in CyREST's swagger documentation for 'session'.

    :param url: an url of the type 'http://' + host + ':' + str(port) + '/' + version + '/'.
    """

    def __init__(self, url):
        self.__url = url + 'commands/session'
        self.___url=url

    def new(self, verbose=False):
        """
        Destroys the current session and creates a new, empty one.

        :param wid: Window ID
        :param verbose: print more
        """

        response=api(url=self.__url+"/new", verbose=verbose)
        return response

    
    def open(self, session_file=None,session_url=None, verbose=False):
        """
        Opens a session from a local file or URL.

        :param session_file: The path to the session file (.cys) to be loaded.
        :param session_url: A URL that provides a session file.
        :param verbose: print more
        """

        PARAMS=set_param(["file", "url"],[session_file, session_url])
        response=api(url=self.__url+"/open", PARAMS=PARAMS, verbose=verbose)
        return response

    
    def save(self, session_file, verbose=False):
        """
        Saves the current session to an existing file, which will be replaced.
        If this is a new session that has not been saved yet, use 'save as'
        instead.

        :param session_file: The path to the file where the current session
        must be saved to.
        :param verbose: print more
        """

        PARAMS={"file":session_file}

        response=api(url=self.__url+"/save", PARAMS=PARAMS, verbose=verbose)
        return response

    
    def save_as(self, session_file, verbose=False):
        """
        Saves the current session as a new file.

        :param session_file: The path to the file where the current session
        must be saved to.
        :param verbose: print more
        """

        PARAMS={"file":session_file}

        response=api(url=self.__url+"/save as", PARAMS=PARAMS, verbose=verbose)
        return response

    def createSessionFile(self, file, verbose=None):
        """
        Saves the current session to a file. If successful, the session file location will be returned.

        :param file: Session file location as an absolute path
        :param verbose: print more

        :returns: 200: successful operation
        """

        PARAMS=set_param(['file'],[file])
        response=api(url=self.___url+'session', PARAMS=PARAMS, method="POST", verbose=verbose)
        return response


    def deleteSession(self, verbose=None):
        """
        This deletes the current session and initializes a new one. A message is returned to indicate the success of the deletion.

        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'session', method="DELETE", verbose=verbose)
        return response


    def getSessionFromFile(self, file, verbose=None):
        """
        Loads a session from a local file and returns the session file name

        :param file: Session file location as an absolute path
        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'session', PARAMS={'file':file}, method="GET", verbose=verbose, parse_params=False)
        return response


    def getSessionName(self, verbose=None):
        """
        Returns the file name for the current Cytoscape session.

        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'session/name', method="GET", verbose=verbose, parse_params=False)
        return response

    def runGarbageCollection(self, verbose=None):
        """
        Manually call Java's System.gc() to free up unused memory. This process happens automatically, but may be useful to call explicitly for testing or evaluation purposes.

        :param verbose: print more

        :returns: 204: Successful Garbage Collection
        """

        response=api(url=self.___url+'gc', method="GET", verbose=verbose, parse_params=False)
        return response


from .base import *

class edge(object):
    """
    cytoscape session interface as shown in CyREST's swagger documentation for 'edge'.

    :param url: an url of the type 'http://' + host + ':' + str(port) + '/' + version + '/'.
    """

    def __init__(self, url):
        self.__url = url + 'commands/edge'

    def create_attribute(self,column=None,listType=None,namespace=None, network=None, atype=None, verbose=False):
        """
        Creates a new edge column.

        :param column (string, optional): Unique name of column
        :param listType (string, optional): Can be one of integer, long, double,
            or string.
        :param namespace (string, optional): Node, Edge, and Network objects
            support the default, local, and hidden namespaces. Root networks
            also support the shared namespace. Custom namespaces may be specified
            by Apps.
        :param network (string, optional): Specifies a network by name, or by
            SUID if the prefix SUID: is used. The keyword CURRENT, or a blank
            value can also be used to specify the current network.
        :param atype (string, optional): Can be one of integer, long, double,
            string, or list.
        :param verbose: print more

        """
        network=check_network(self,network,verbose=verbose)
        PARAMS=set_param(["column","listType","namespace","network","type"],[column,listType,namespace,network,atype])
        response=api(url=self.__url+"/create attribute", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response

    def get(self,edge=None,network=None,sourceNode=None, targetNode=None, atype=None, verbose=False):
        """
        Returns the SUID of an edge that matches the passed parameters. If
        multiple edges are found, only one will be returned, and a warning will
        be reported in the Cytoscape Task History dialog.

        :param edge (string, optional): Selects an edge by name, or, if the
            parameter has the prefix suid:, selects an edge by SUID. If this
            parameter is set, all other edge matching parameters are ignored.
        :param network (string, optional): Specifies a network by name, or by
            SUID if the prefix SUID: is used. The keyword CURRENT, or a blank
            value can also be used to specify the current network.
        :param sourceNode (string, optional): Selects a node by name, or, if
            the parameter has the prefix suid:, selects a node by SUID. Specifies
            that the edge matched must have this node as its source. This parameter
            must be used with the targetNode parameter to produce results.
        :param targetNode (string, optional): Selects a node by name, or, if
            the parameter has the prefix suid:, selects a node by SUID. Specifies
            that the edge matched must have this node as its target. This parameter
            must be used with the sourceNode parameter to produce results.
        :param atype (string, optional): Specifies that the edge matched must
            be of the specified type. This parameter must be used with the
            sourceNode and targetNode parameters to produce results.
        :param verbose: print more

        :returns: {"columnName": columnName }

        """
        network=check_network(self,network,verbose=verbose)
        PARAMS=set_param(["edge","network","sourceNode","targetNode","type"],[edge,network,sourceNode,targetNode,atype])
        response=api(url=self.__url+"/get", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response

    def get_attribute(self,columnList=None,edgeList=None,namespace=None, network=None, verbose=False):
        """
        Returns the attributes for the edges passed as parameters.

        :param columnList (string, optional): A list of column names, separated
            by commas.
        :param edgeList (string, optional): Specifies a list of edges. The
            keywords all, selected, or unselected can be used to specify edges
            by their selection state. The pattern COLUMN:VALUE sets this parameter
            to any rows that contain the specified column value; if the COLUMN
            prefix is not used, the NAME column is matched by default. A list
            of COLUMN:VALUE pairs of the format COLUMN1:VALUE1,COLUMN2:VALUE2,...
            can be used to match multiple values.
        :param namespace (string, optional): Node, Edge, and Network objects
            support the default, local, and hidden namespaces. Root networks
            also support the shared namespace. Custom namespaces may be specified
            by Apps.
        :param network (string, optional): Specifies a network by name, or by
            SUID if the prefix SUID: is used. The keyword CURRENT, or a blank
            value can also be used to specify the current network.
        :param verbose: print more

        :returns: eg. [ {"name": "TAT (pp) O60563"}, {"name": "TAT (pp) O60563"}]

        """
        network=check_network(self,network,verbose=verbose)
        PARAMS=set_param(["columnList","edgeList","namespace","network"],[columnList,edgeList,namespace,network])
        response=api(url=self.__url+"/get attribute", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response

    def get_properties(self,edgeList=None,network=None,propertyList=None, verbose=False):
        """
        Returns the visual properties for the edges that match the passed parameters.

        :param edgeList (string, optional): Specifies a list of edges. The
            keywords all, selected, or unselected can be used to specify edges
            by their selection state. The pattern COLUMN:VALUE sets this parameter
            to any rows that contain the specified column value; if the COLUMN
            prefix is not used, the NAME column is matched by default. A list
            of COLUMN:VALUE pairs of the format COLUMN1:VALUE1,COLUMN2:VALUE2,...
            can be used to match multiple values.
        :param network (string, optional): Specifies a network by name, or by
            SUID if the prefix SUID: is used. The keyword CURRENT, or a blank
            value can also be used to specify the current network.
        :param propertyList (string, optional): A list of property names
            separated by commas.
        :param verbose: print more

        :returns: eg. [
                        {
                          "SUID": 1647,
                          "visualProperties": [
                            {
                              "visualProperty": "EDGE_PAINT",
                              "value": "#808080"
                            },
                            {
                              "visualProperty": "EDGE_VISIBLE",
                              "value": true
                            }
                          ]
                        },
                        ...
                        ]
        """
        network=check_network(self,network,verbose=verbose)
        PARAMS=set_param(["edgeList","network","propertyList"],[edgeList,network,propertyList])
        response=api(url=self.__url+"/get properties", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response

    def list(self, edgeList=None, network=None, verbose=False):
        """
        Returns a list of the edge SUIDs associated with the passed network parameter.

        :param edgeList (string, optional): Specifies a list of edges. The
            keywords all, selected, or unselected can be used to specify edges
            by their selection state. The pattern COLUMN:VALUE sets this parameter
            to any rows that contain the specified column value; if the COLUMN
            prefix is not used, the NAME column is matched by default. A list of
            COLUMN:VALUE pairs of the format COLUMN1:VALUE1,COLUMN2:VALUE2,...
            can be used to match multiple values.
        :param network (string, optional): Specifies a network by name, or by
            SUID if the prefix SUID: is used. The keyword CURRENT, or a blank
            value can also be used to specify the current network.
        :param verbose: print more

        :returns: eg. [ 772, 773, .. ]
        """
        network=check_network(self,network,verbose=verbose)
        PARAMS=set_param(["edgeList","network"],[edgeList,network])
        response=api(url=self.__url+"/list", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response

    def list_attributes(self,columnList=None,edgeList=None,namespace=None, network=None, verbose=False):
        """
        Returns the attributes for the edges passed as parameters.

        :param columnList (string, optional): A list of column names, separated
            by commas.
        :param edgeList (string, optional): Specifies a list of edges. The
            keywords all, selected, or unselected can be used to specify edges
            by their selection state. The pattern COLUMN:VALUE sets this parameter
            to any rows that contain the specified column value; if the COLUMN
            prefix is not used, the NAME column is matched by default. A list
            of COLUMN:VALUE pairs of the format COLUMN1:VALUE1,COLUMN2:VALUE2,...
            can be used to match multiple values.
        :param namespace (string, optional): Node, Edge, and Network objects
            support the default, local, and hidden namespaces. Root networks
            also support the shared namespace. Custom namespaces may be specified
            by Apps.
        :param network (string, optional): Specifies a network by name, or by
            SUID if the prefix SUID: is used. The keyword CURRENT, or a blank
            value can also be used to specify the current network.
        :param verbose: print more

        :returns:  eg. [
                        "SUID",
                        "shared name",
                        "shared interaction",
                        "name",
                        "selected",
                        "interaction",
                        "Annotation",
                        .. ]
        """
        network=check_network(self,network,verbose=verbose)
        PARAMS=set_param(["columnList","edgeList","namespace","network"],[columnList,edgeList,namespace,network])
        response=api(url=self.__url+"/list attributes", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response

    def list_properties(self,columnList=None,edgeList=None,namespace=None, network=None, verbose=False):
        """
        Returns a list of visual properties available for edges.

        :param network (string, optional): Specifies a network by name, or by
            SUID if the prefix SUID: is used. The keyword CURRENT, or a blank
            value can also be used to specify the current network.
        :param verbose: print more

        :returns:  eg. [
                        "Label Font Size",
                        "Source Arrow Size",
                        "Stroke Color (Unselected)",
                        "Source Arrow Selected Paint",
                        "Label Font Face",
                        "Target Arrow Unselected Paint",
                        "Transparency",
                        "Source Arrow Unselected Paint",
                        "Tooltip",
                        .. ]
        """
        network=check_network(self,network,verbose=verbose)
        PARAMS=set_param(["network"],[network])
        response=api(url=self.__url+"/list properties", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response

    def rename(self, edge=None, network=None, newName=None, verbose=False):
        """
        Sets the value of the name column for the passed edge.

        :param edge (string, optional): Selects an edge by name, or, if the
            parameter has the prefix suid:, selects an edge by SUID.
        :param network (string, optional): Specifies a network by name, or by
            SUID if the prefix SUID: is used. The keyword CURRENT, or a blank
            value can also be used to specify the current network.
        :param newName (string, optional): New name of the edge
        :param verbose: print more

        """
        network=check_network(self,network,verbose=verbose)
        PARAMS=set_param(["edge","network","newName"],[edge,network,newName])
        response=api(url=self.__url+"/rename", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response

    def set_attribute(self, columnList=None, edgeList=None, namespace=None, network=None, valueList=None, verbose=False):
        """
        Sets the value of a specified column for the passed edge or set of edges.

        :param columnList (string, optional): A list of column names, separated
            by commas.
        :param edgeList (string, optional): Specifies a list of edges. The
            keywords all, selected, or unselected can be used to specify edges
            by their selection state. The pattern COLUMN:VALUE sets this parameter
            to any rows that contain the specified column value; if the COLUMN
            prefix is not used, the NAME column is matched by default. A list
            of COLUMN:VALUE pairs of the format COLUMN1:VALUE1,COLUMN2:VALUE2,...
            can be used to match multiple values.
        :param namespace (string, optional): Node, Edge, and Network objects
            support the default, local, and hidden namespaces. Root networks
            also support the shared namespace. Custom namespaces may be specified
            by Apps.
        :param network (string, optional): Specifies a network by name, or by
            SUID if the prefix SUID: is used. The keyword CURRENT, or a blank
            value can also be used to specify the current network.
        :param valueList (string, optional): A list of values, separated by
            commas. List values can be included using the format [value1,value2].
        :param verbose: print more

        """
        network=check_network(self,network,verbose=verbose)
        PARAMS=set_param(["columnList","edgeList","namespace","network", "valueList"],[columnList, edgeList, namespace, network, valueList])
        response=api(url=self.__url+"/set attribute", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response

    def set_properties(self, edgeList=None, network=None, propertyList=None, valueList=None, verbose=False):
        """
        Sets the value of a specified property for the passed edge or set of edges.

        :param edgeList (string, optional): Specifies a list of edges. The
            keywords all, selected, or unselected can be used to specify edges
            by their selection state. The pattern COLUMN:VALUE sets this parameter
            to any rows that contain the specified column value; if the COLUMN
            prefix is not used, the NAME column is matched by default. A list of
            COLUMN:VALUE pairs of the format COLUMN1:VALUE1,COLUMN2:VALUE2,...
            can be used to match multiple values.
        :param network (string, optional): Specifies a network by name, or by
            SUID if the prefix SUID: is used. The keyword CURRENT, or a blank
            value can also be used to specify the current network.
        :param propertyList (string, optional): A list of property names
            separated by commas.
        :param valueList (string, optional): A list of values separated by commas.
        :param verbose: print more
        """
        network=check_network(self,network,verbose=verbose)
        PARAMS=set_param(["edgeList","network","propertyList","valueList"],[edgeList, network, propertyList, valueList])
        response=api(url=self.__url+"/set properties", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response
